@@ -1,56 +1,19 @@
 /*
- * Copyright (C) 2009-2010, Google Inc.
- * and other copyright owners as documented in the project's IP log.
+ * Copyright (C) 2009-2010, Google Inc. and others
  *
- * This program and the accompanying materials are made available
- * under the terms of the Eclipse Distribution License v1.0 which
- * accompanies this distribution, is reproduced below, and is
- * available at http://www.eclipse.org/org/documents/edl-v10.php
+ * This program and the accompanying materials are made available under the
+ * terms of the Eclipse Distribution License v. 1.0 which is available at
+ * https://www.eclipse.org/org/documents/edl-v10.php.
  *
- * All rights reserved.
- *
- * Redistribution and use in source and binary forms, with or
- * without modification, are permitted provided that the following
- * conditions are met:
- *
- * - Redistributions of source code must retain the above copyright
- *   notice, this list of conditions and the following disclaimer.
- *
- * - Redistributions in binary form must reproduce the above
- *   copyright notice, this list of conditions and the following
- *   disclaimer in the documentation and/or other materials provided
- *   with the distribution.
- *
- * - Neither the name of the Eclipse Foundation, Inc. nor the
- *   names of its contributors may be used to endorse or promote
- *   products derived from this software without specific prior
- *   written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
- * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
- * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
- * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
- * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
- * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
- * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
- * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
- * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ * SPDX-License-Identifier: BSD-3-Clause
  */
 
 package org.eclipse.jgit.http.server;
 
-import static javax.servlet.http.HttpServletResponse.SC_BAD_REQUEST;
 import static javax.servlet.http.HttpServletResponse.SC_FORBIDDEN;
 import static javax.servlet.http.HttpServletResponse.SC_INTERNAL_SERVER_ERROR;
 import static javax.servlet.http.HttpServletResponse.SC_UNAUTHORIZED;
 import static javax.servlet.http.HttpServletResponse.SC_UNSUPPORTED_MEDIA_TYPE;
-import static org.eclipse.jgit.http.server.ClientVersionUtil.hasChunkedEncodingRequestBug;
-import static org.eclipse.jgit.http.server.ClientVersionUtil.hasPushStatusBug;
-import static org.eclipse.jgit.http.server.ClientVersionUtil.parseVersion;
 import static org.eclipse.jgit.http.server.GitSmartHttpTools.RECEIVE_PACK;
 import static org.eclipse.jgit.http.server.GitSmartHttpTools.RECEIVE_PACK_REQUEST_TYPE;
 import static org.eclipse.jgit.http.server.GitSmartHttpTools.RECEIVE_PACK_RESULT_TYPE;
@@ -75,6 +38,7 @@ import javax.servlet.http.HttpServlet;
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
 
+import org.eclipse.jgit.annotations.Nullable;
 import org.eclipse.jgit.errors.CorruptObjectException;
 import org.eclipse.jgit.errors.PackProtocolException;
 import org.eclipse.jgit.errors.UnpackException;
@@ -165,7 +129,13 @@ class ReceivePackServlet extends HttpServlet {
 		}
 	}
 
-	/** {@inheritDoc} */
+	@Nullable
+	private final ReceivePackErrorHandler handler;
+
+	ReceivePackServlet(@Nullable ReceivePackErrorHandler handler) {
+		this.handler = handler;
+	}
+
 	@Override
 	public void doPost(final HttpServletRequest req,
 			final HttpServletResponse rsp) throws IOException {
@@ -174,13 +144,6 @@ class ReceivePackServlet extends HttpServlet {
 			return;
 		}
 
-		int[] version = parseVersion(req.getHeader(HDR_USER_AGENT));
-		if (hasChunkedEncodingRequestBug(version, req)) {
-			GitSmartHttpTools.sendError(req, rsp, SC_BAD_REQUEST, "\n\n"
-					+ HttpServerText.get().clientHas175ChunkedEncodingBug);
-			return;
-		}
-
 		SmartOutputStream out = new SmartOutputStream(req, rsp, false) {
 			@Override
 			public void flush() throws IOException {
@@ -189,35 +152,42 @@ class ReceivePackServlet extends HttpServlet {
 		};
 
 		ReceivePack rp = (ReceivePack) req.getAttribute(ATTRIBUTE_HANDLER);
-		try {
-			rp.setBiDirectionalPipe(false);
-			rp.setEchoCommandFailures(hasPushStatusBug(version));
-			rsp.setContentType(RECEIVE_PACK_RESULT_TYPE);
+		rp.setBiDirectionalPipe(false);
+		rsp.setContentType(RECEIVE_PACK_RESULT_TYPE);
 
-			rp.receive(getInputStream(req), out, null);
-			out.close();
-		} catch (CorruptObjectException e ) {
-			// This should be already reported to the client.
-			getServletContext().log(MessageFormat.format(
-					HttpServerText.get().receivedCorruptObject,
-					e.getMessage(),
-					ServletUtils.identify(rp.getRepository())));
-			consumeRequestBody(req);
-			out.close();
+		if (handler != null) {
+			handler.receive(req, rsp, () -> {
+				rp.receiveWithExceptionPropagation(getInputStream(req), out,
+						null);
+				out.close();
+			});
+		} else {
+			try {
+				rp.receive(getInputStream(req), out, null);
+				out.close();
+			} catch (CorruptObjectException e ) {
+				// This should be already reported to the client.
+				getServletContext().log(MessageFormat.format(
+						HttpServerText.get().receivedCorruptObject,
+						e.getMessage(),
+						ServletUtils.identify(rp.getRepository())));
+				consumeRequestBody(req);
+				out.close();
 
-		} catch (UnpackException | PackProtocolException e) {
-			// This should be already reported to the client.
-			log(rp.getRepository(), e.getCause());
-			consumeRequestBody(req);
-			out.close();
+			} catch (UnpackException | PackProtocolException e) {
+				// This should be already reported to the client.
+				log(rp.getRepository(), e.getCause());
+				consumeRequestBody(req);
+				out.close();
 
-		} catch (Throwable e) {
-			log(rp.getRepository(), e);
-			if (!rsp.isCommitted()) {
-				rsp.reset();
-				sendError(req, rsp, SC_INTERNAL_SERVER_ERROR);
+			} catch (Throwable e) {
+				log(rp.getRepository(), e);
+				if (!rsp.isCommitted()) {
+					rsp.reset();
+					sendError(req, rsp, SC_INTERNAL_SERVER_ERROR);
+				}
+				return;
 			}
-			return;
 		}
 	}
 
