@@ -3,46 +3,13 @@
  * Copyright (C) 2008-2010, Google Inc.
  * Copyright (C) 2008, Marek Zawirski <marek.zawirski@gmail.com>
  * Copyright (C) 2008, Shawn O. Pearce <spearce@spearce.org>
- * Copyright (C) 2010, Sasa Zivkov <sasa.zivkov@sap.com>
- * and other copyright owners as documented in the project's IP log.
+ * Copyright (C) 2010, Sasa Zivkov <sasa.zivkov@sap.com> and others
  *
- * This program and the accompanying materials are made available
- * under the terms of the Eclipse Distribution License v1.0 which
- * accompanies this distribution, is reproduced below, and is
- * available at http://www.eclipse.org/org/documents/edl-v10.php
+ * This program and the accompanying materials are made available under the
+ * terms of the Eclipse Distribution License v. 1.0 which is available at
+ * https://www.eclipse.org/org/documents/edl-v10.php.
  *
- * All rights reserved.
- *
- * Redistribution and use in source and binary forms, with or
- * without modification, are permitted provided that the following
- * conditions are met:
- *
- * - Redistributions of source code must retain the above copyright
- *   notice, this list of conditions and the following disclaimer.
- *
- * - Redistributions in binary form must reproduce the above
- *   copyright notice, this list of conditions and the following
- *   disclaimer in the documentation and/or other materials provided
- *   with the distribution.
- *
- * - Neither the name of the Eclipse Foundation, Inc. nor the
- *   names of its contributors may be used to endorse or promote
- *   products derived from this software without specific prior
- *   written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
- * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
- * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
- * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
- * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
- * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
- * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
- * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
- * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ * SPDX-License-Identifier: BSD-3-Clause
  */
 
 package org.eclipse.jgit.pgm;
@@ -66,11 +33,18 @@ abstract class AbstractFetchCommand extends TextBuiltin {
 	@Option(name = "--verbose", aliases = { "-v" }, usage = "usage_beMoreVerbose")
 	private boolean verbose;
 
-	protected void showFetchResult(final FetchResult r) throws IOException {
-		ObjectReader reader = db.newObjectReader();
-		try {
+	/**
+	 * Show fetch result.
+	 *
+	 * @param r
+	 *            a {@link org.eclipse.jgit.transport.FetchResult} object.
+	 * @throws java.io.IOException
+	 *             if any.
+	 */
+	protected void showFetchResult(FetchResult r) throws IOException {
+		try (ObjectReader reader = db.newObjectReader()) {
 			boolean shownURI = false;
-			for (final TrackingRefUpdate u : r.getTrackingRefUpdates()) {
+			for (TrackingRefUpdate u : r.getTrackingRefUpdates()) {
 				if (!verbose && u.getResult() == RefUpdate.Result.NO_CHANGE)
 					continue;
 
@@ -89,10 +63,11 @@ abstract class AbstractFetchCommand extends TextBuiltin {
 						src, dst);
 				outw.println();
 			}
-		} finally {
-			reader.release();
 		}
 		showRemoteMessages(errw, r.getMessages());
+		for (FetchResult submoduleResult : r.submoduleResults().values()) {
+			showFetchResult(submoduleResult);
+		}
 	}
 
 	static void showRemoteMessages(ThrowingPrintWriter writer, String pkt) throws IOException {
@@ -132,20 +107,20 @@ abstract class AbstractFetchCommand extends TextBuiltin {
 			final TrackingRefUpdate u) {
 		final RefUpdate.Result r = u.getResult();
 		if (r == RefUpdate.Result.LOCK_FAILURE)
-			return "[lock fail]";
+			return "[lock fail]"; //$NON-NLS-1$
 		if (r == RefUpdate.Result.IO_FAILURE)
-			return "[i/o error]";
+			return "[i/o error]"; //$NON-NLS-1$
 		if (r == RefUpdate.Result.REJECTED)
-			return "[rejected]";
+			return "[rejected]"; //$NON-NLS-1$
 		if (ObjectId.zeroId().equals(u.getNewObjectId()))
-			return "[deleted]";
+			return "[deleted]"; //$NON-NLS-1$
 
 		if (r == RefUpdate.Result.NEW) {
 			if (u.getRemoteName().startsWith(Constants.R_HEADS))
-				return "[new branch]";
+				return "[new branch]"; //$NON-NLS-1$
 			else if (u.getLocalName().startsWith(Constants.R_TAGS))
-				return "[new tag]";
-			return "[new]";
+				return "[new tag]"; //$NON-NLS-1$
+			return "[new]"; //$NON-NLS-1$
 		}
 
 		if (r == RefUpdate.Result.FORCED) {
@@ -161,7 +136,7 @@ abstract class AbstractFetchCommand extends TextBuiltin {
 		}
 
 		if (r == RefUpdate.Result.NO_CHANGE)
-			return "[up to date]";
+			return "[up to date]"; //$NON-NLS-1$
 		return "[" + r.name() + "]"; //$NON-NLS-1$//$NON-NLS-2$
 	}
 
@@ -173,7 +148,7 @@ abstract class AbstractFetchCommand extends TextBuiltin {
 		}
 	}
 
-	private static char shortTypeOf(final RefUpdate.Result r) {
+	private static char shortTypeOf(RefUpdate.Result r) {
 		if (r == RefUpdate.Result.LOCK_FAILURE)
 			return '!';
 		if (r == RefUpdate.Result.IO_FAILURE)
