@@ -10,6 +10,7 @@
 
 package org.eclipse.jgit.pgm;
 
+import org.eclipse.jgit.api.GarbageCollectCommand;
 import org.eclipse.jgit.api.Git;
 import org.eclipse.jgit.api.errors.GitAPIException;
 import org.eclipse.jgit.lib.TextProgressMonitor;
@@ -21,20 +22,24 @@ class Gc extends TextBuiltin {
 	private boolean aggressive;
 
 	@Option(name = "--preserve-oldpacks", usage = "usage_PreserveOldPacks")
-	private boolean preserveOldPacks;
+	private Boolean preserveOldPacks;
 
 	@Option(name = "--prune-preserved", usage = "usage_PrunePreserved")
-	private boolean prunePreserved;
+	private Boolean prunePreserved;
 
-	/** {@inheritDoc} */
 	@Override
 	protected void run() {
 		Git git = Git.wrap(db);
 		try {
-			git.gc().setAggressive(aggressive)
-					.setPreserveOldPacks(preserveOldPacks)
-					.setPrunePreserved(prunePreserved)
-					.setProgressMonitor(new TextProgressMonitor(errw)).call();
+			GarbageCollectCommand command = git.gc().setAggressive(aggressive)
+					.setProgressMonitor(new TextProgressMonitor(errw));
+			if (preserveOldPacks != null) {
+				command.setPreserveOldPacks(preserveOldPacks.booleanValue());
+			}
+			if (prunePreserved != null) {
+				command.setPrunePreserved(prunePreserved.booleanValue());
+			}
+			command.call();
 		} catch (GitAPIException e) {
 			throw die(e.getMessage(), e);
 		}
