@@ -34,6 +34,8 @@ import org.eclipse.jgit.lib.RefUpdate;
 import org.eclipse.jgit.lib.Repository;
 import org.eclipse.jgit.lib.RepositoryState;
 import org.eclipse.jgit.lib.StoredConfig;
+import org.eclipse.jgit.merge.ContentMergeStrategy;
+import org.eclipse.jgit.merge.MergeStrategy;
 import org.eclipse.jgit.revwalk.RevCommit;
 import org.eclipse.jgit.revwalk.RevSort;
 import org.eclipse.jgit.revwalk.RevWalk;
@@ -153,6 +155,75 @@ public class PullCommandTest extends RepositoryTestCase {
 				.getRepositoryState());
 	}
 
+	@Test
+	public void testPullConflictTheirs() throws Exception {
+		PullResult res = target.pull().call();
+		// nothing to update since we don't have different data yet
+		assertTrue(res.getFetchResult().getTrackingRefUpdates().isEmpty());
+		assertTrue(res.getMergeResult().getMergeStatus()
+				.equals(MergeStatus.ALREADY_UP_TO_DATE));
+
+		assertFileContentsEqual(targetFile, "Hello world");
+
+		// change the source file
+		writeToFile(sourceFile, "Source change");
+		source.add().addFilepattern("SomeFile.txt").call();
+		source.commit().setMessage("Source change in remote").call();
+
+		// change the target file
+		writeToFile(targetFile, "Target change");
+		target.add().addFilepattern("SomeFile.txt").call();
+		target.commit().setMessage("Target change in local").call();
+
+		res = target.pull().setStrategy(MergeStrategy.THEIRS).call();
+
+		assertTrue(res.isSuccessful());
+		assertFileContentsEqual(targetFile, "Source change");
+		assertEquals(RepositoryState.SAFE,
+				target.getRepository().getRepositoryState());
+		assertTrue(target.status().call().isClean());
+	}
+
+	@Test
+	public void testPullConflictXtheirs() throws Exception {
+		PullResult res = target.pull().call();
+		// nothing to update since we don't have different data yet
+		assertTrue(res.getFetchResult().getTrackingRefUpdates().isEmpty());
+		assertTrue(res.getMergeResult().getMergeStatus()
+				.equals(MergeStatus.ALREADY_UP_TO_DATE));
+
+		assertFileContentsEqual(targetFile, "Hello world");
+
+		// change the source file
+		writeToFile(sourceFile, "a\nHello\nb\n");
+		source.add().addFilepattern("SomeFile.txt").call();
+		source.commit().setMessage("Multi-line change in remote").call();
+
+		// Pull again
+		res = target.pull().call();
+		assertTrue(res.isSuccessful());
+		assertFileContentsEqual(targetFile, "a\nHello\nb\n");
+
+		// change the source file
+		writeToFile(sourceFile, "a\nSource change\nb\n");
+		source.add().addFilepattern("SomeFile.txt").call();
+		source.commit().setMessage("Source change in remote").call();
+
+		// change the target file
+		writeToFile(targetFile, "a\nTarget change\nb\nc\n");
+		target.add().addFilepattern("SomeFile.txt").call();
+		target.commit().setMessage("Target change in local").call();
+
+		res = target.pull().setContentMergeStrategy(ContentMergeStrategy.THEIRS)
+				.call();
+
+		assertTrue(res.isSuccessful());
+		assertFileContentsEqual(targetFile, "a\nSource change\nb\nc\n");
+		assertEquals(RepositoryState.SAFE,
+				target.getRepository().getRepositoryState());
+		assertTrue(target.status().call().isClean());
+	}
+
 	@Test
 	public void testPullWithUntrackedStash() throws Exception {
 		target.pull().call();
@@ -307,44 +378,59 @@ public class PullCommandTest extends RepositoryTestCase {
 	}
 
 	private enum TestPullMode {
-		MERGE, REBASE, REBASE_PREASERVE
+		MERGE, REBASE, REBASE_MERGES
 	}
 
 	@Test
 	/** global rebase config should be respected */
-	public void testPullWithRebasePreserve1Config() throws Exception {
+	public void testPullWithRebaseMerges1Config() throws Exception {
+		Callable<PullResult> setup = () -> {
+			StoredConfig config = dbTarget.getConfig();
+			config.setString("pull", null, "rebase", "merges");
+			config.save();
+			return target.pull().call();
+		};
+		doTestPullWithRebase(setup, TestPullMode.REBASE_MERGES);
+	}
+
+	@Test
+	/**
+	 * global rebase config using old "preserve" value which was renamed to
+	 * "merges" should be respected to ensure backwards compatibility
+	 */
+	public void testPullWithRebaseMerges1ConfigAlias() throws Exception {
 		Callable<PullResult> setup = () -> {
 			StoredConfig config = dbTarget.getConfig();
 			config.setString("pull", null, "rebase", "preserve");
 			config.save();
 			return target.pull().call();
 		};
-		doTestPullWithRebase(setup, TestPullMode.REBASE_PREASERVE);
+		doTestPullWithRebase(setup, TestPullMode.REBASE_MERGES);
 	}
 
 	@Test
 	/** the branch-local config should win over the global config */
-	public void testPullWithRebasePreserveConfig2() throws Exception {
+	public void testPullWithRebaseMergesConfig2() throws Exception {
 		Callable<PullResult> setup = () -> {
 			StoredConfig config = dbTarget.getConfig();
 			config.setString("pull", null, "rebase", "false");
-			config.setString("branch", "master", "rebase", "preserve");
+			config.setString("branch", "master", "rebase", "merges");
 			config.save();
 			return target.pull().call();
 		};
-		doTestPullWithRebase(setup, TestPullMode.REBASE_PREASERVE);
+		doTestPullWithRebase(setup, TestPullMode.REBASE_MERGES);
 	}
 
 	@Test
 	/** the branch-local config should be respected */
-	public void testPullWithRebasePreserveConfig3() throws Exception {
+	public void testPullWithRebaseMergesConfig3() throws Exception {
 		Callable<PullResult> setup = () -> {
 			StoredConfig config = dbTarget.getConfig();
-			config.setString("branch", "master", "rebase", "preserve");
+			config.setString("branch", "master", "rebase", "merges");
 			config.save();
 			return target.pull().call();
 		};
-		doTestPullWithRebase(setup, TestPullMode.REBASE_PREASERVE);
+		doTestPullWithRebase(setup, TestPullMode.REBASE_MERGES);
 	}
 
 	@Test
@@ -364,7 +450,7 @@ public class PullCommandTest extends RepositoryTestCase {
 	public void testPullWithRebaseConfig2() throws Exception {
 		Callable<PullResult> setup = () -> {
 			StoredConfig config = dbTarget.getConfig();
-			config.setString("pull", null, "rebase", "preserve");
+			config.setString("pull", null, "rebase", "merges");
 			config.setString("branch", "master", "rebase", "true");
 			config.save();
 			return target.pull().call();
@@ -472,7 +558,7 @@ public class PullCommandTest extends RepositoryTestCase {
 				assertEquals(sourceCommit, next.getParent(1));
 				// since both parents are known do no further checks here
 			} else {
-				if (expectedPullMode == TestPullMode.REBASE_PREASERVE) {
+				if (expectedPullMode == TestPullMode.REBASE_MERGES) {
 					next = rw.next();
 					assertEquals(2, next.getParentCount());
 				}
@@ -496,6 +582,7 @@ public class PullCommandTest extends RepositoryTestCase {
 	public void setUp() throws Exception {
 		super.setUp();
 		dbTarget = createWorkRepository();
+		addRepoToClose(dbTarget);
 		source = new Git(db);
 		target = new Git(dbTarget);
 
