@@ -13,8 +13,10 @@ import static java.nio.charset.StandardCharsets.UTF_8;
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertFalse;
 import static org.junit.Assert.assertNull;
+import static org.junit.Assert.assertThrows;
 import static org.junit.Assert.assertTrue;
 import static org.junit.Assert.fail;
+import static org.junit.Assume.assumeTrue;
 
 import java.io.BufferedReader;
 import java.io.ByteArrayInputStream;
@@ -31,6 +33,7 @@ import org.eclipse.jgit.api.Git;
 import org.eclipse.jgit.api.errors.GitAPIException;
 import org.eclipse.jgit.api.errors.InvalidRefNameException;
 import org.eclipse.jgit.api.errors.InvalidRemoteException;
+import org.eclipse.jgit.gitrepo.RepoCommand.ManifestErrorException;
 import org.eclipse.jgit.gitrepo.RepoCommand.RemoteFile;
 import org.eclipse.jgit.internal.JGitText;
 import org.eclipse.jgit.junit.JGitTestUtil;
@@ -46,6 +49,8 @@ import org.eclipse.jgit.revwalk.RevCommit;
 import org.eclipse.jgit.storage.file.FileBasedConfig;
 import org.eclipse.jgit.treewalk.TreeWalk;
 import org.eclipse.jgit.util.FS;
+import org.eclipse.jgit.util.IO;
+import org.eclipse.jgit.util.RawParseUtils;
 import org.junit.Test;
 
 public class RepoCommandTest extends RepositoryTestCase {
@@ -544,24 +549,29 @@ public class RepoCommandTest extends RepositoryTestCase {
 		// The original file should exist
 		File hello = new File(localDb.getWorkTree(), "foo/hello.txt");
 		assertTrue("The original file should exist", hello.exists());
-		assertFalse("The original file should not be executable",
-				hello.canExecute());
+		if (FS.DETECTED.supportsExecute()) {
+			assertFalse("The original file should not be executable",
+					FS.DETECTED.canExecute(hello));
+		}
 		assertContents(hello.toPath(), "master world");
 		// The dest file should also exist
 		hello = new File(localDb.getWorkTree(), "Hello");
 		assertTrue("The destination file should exist", hello.exists());
-		assertFalse("The destination file should not be executable",
-				hello.canExecute());
+		if (FS.DETECTED.supportsExecute()) {
+			assertFalse("The destination file should not be executable",
+					FS.DETECTED.canExecute(hello));
+		}
 		assertContents(hello.toPath(), "master world");
 	}
 
 	@Test
 	public void testRepoManifestCopyFile_executable() throws Exception {
+		assumeTrue(FS.DETECTED.supportsExecute());
 		try (Git git = new Git(defaultDb)) {
 			git.checkout().setName("master").call();
 			File f = JGitTestUtil.writeTrashFile(defaultDb, "hello.sh",
 					"content of the executable file");
-			f.setExecutable(true);
+			FS.DETECTED.setExecute(f, true);
 			git.add().addFilepattern("hello.sh").call();
 			git.commit().setMessage("Add binary file").call();
 		}
@@ -586,7 +596,8 @@ public class RepoCommandTest extends RepositoryTestCase {
 		// The original file should exist and be an executable
 		File hello = new File(localDb.getWorkTree(), "foo/hello.sh");
 		assertTrue("The original file should exist", hello.exists());
-		assertTrue("The original file must be executable", hello.canExecute());
+		assertTrue("The original file must be executable",
+				FS.DETECTED.canExecute(hello));
 		try (BufferedReader reader = Files.newBufferedReader(hello.toPath(),
 				UTF_8)) {
 			String content = reader.readLine();
@@ -598,7 +609,7 @@ public class RepoCommandTest extends RepositoryTestCase {
 		hello = new File(localDb.getWorkTree(), "copy-hello.sh");
 		assertTrue("The destination file should exist", hello.exists());
 		assertTrue("The destination file must be executable",
-				hello.canExecute());
+				FS.DETECTED.canExecute(hello));
 		try (BufferedReader reader = Files.newBufferedReader(hello.toPath(),
 				UTF_8)) {
 			String content = reader.readLine();
@@ -749,9 +760,55 @@ public class RepoCommandTest extends RepositoryTestCase {
 			String gitlink = localDb.resolve(Constants.HEAD + ":foo").name();
 			assertEquals("The gitlink is same as remote head",
 					oldCommitId.name(), gitlink);
+
+			File dotmodules = new File(localDb.getWorkTree(),
+					Constants.DOT_GIT_MODULES);
+			assertTrue(dotmodules.exists());
+			// The .gitmodules file should have "branch" lines
+			String gitModulesContents = RawParseUtils
+					.decode(IO.readFully(dotmodules));
+			assertTrue(gitModulesContents.contains("branch = branch"));
 		}
 	}
 
+	@Test
+	public void testRevisionBare_ignoreTags() throws Exception {
+		Repository remoteDb = createBareRepository();
+		Repository tempDb = createWorkRepository();
+
+		StringBuilder xmlContent = new StringBuilder();
+		xmlContent.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n")
+				.append("<manifest>")
+				.append("<remote name=\"remote1\" fetch=\".\" />")
+				.append("<default revision=\"").append("refs/tags/" + TAG)
+				.append("\" remote=\"remote1\" />")
+				.append("<project path=\"foo\" name=\"")
+				.append(defaultUri)
+				.append("\" />").append("</manifest>");
+		JGitTestUtil.writeTrashFile(tempDb, "manifest.xml",
+				xmlContent.toString());
+		RepoCommand command = new RepoCommand(remoteDb);
+		command.setPath(
+				tempDb.getWorkTree().getAbsolutePath() + "/manifest.xml")
+				.setURI(rootUri).call();
+		// Clone it
+		File directory = createTempDirectory("testReplaceManifestBare");
+		File dotmodules;
+		try (Repository localDb = Git.cloneRepository().setDirectory(directory)
+				.setURI(remoteDb.getDirectory().toURI().toString()).call()
+				.getRepository()) {
+			dotmodules = new File(localDb.getWorkTree(),
+					Constants.DOT_GIT_MODULES);
+			assertTrue(dotmodules.exists());
+		}
+
+		// The .gitmodules file should not have "branch" lines
+		String gitModulesContents = RawParseUtils
+				.decode(IO.readFully(dotmodules));
+		assertFalse(gitModulesContents.contains("branch"));
+		assertTrue(gitModulesContents.contains("ref = refs/tags/" + TAG));
+	}
+
 	@Test
 	public void testCopyFileBare() throws Exception {
 		Repository remoteDb = createBareRepository();
@@ -1282,6 +1339,28 @@ public class RepoCommandTest extends RepositoryTestCase {
 		}
 	}
 
+	@Test
+	public void testInvalidPath() throws Exception {
+		Repository remoteDb = createBareRepository();
+		Repository tempDb = createWorkRepository();
+
+		StringBuilder xmlContent = new StringBuilder();
+		xmlContent.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n")
+				.append("<manifest>")
+				.append("<remote name=\"remote1\" fetch=\".\" />")
+				.append("<default revision=\"master\" remote=\"remote1\" />")
+				.append("<project path=\".\" ").append("name=\"")
+				.append(defaultUri).append("\" />").append("</manifest>");
+		JGitTestUtil.writeTrashFile(tempDb, "manifest.xml",
+				xmlContent.toString());
+
+		RepoCommand command = new RepoCommand(remoteDb);
+		command.setPath(
+				tempDb.getWorkTree().getAbsolutePath() + "/manifest.xml")
+				.setURI(rootUri).setRecommendShallow(true);
+		assertThrows(ManifestErrorException.class, () -> command.call());
+	}
+
 	private void resolveRelativeUris() {
 		// Find the longest common prefix ends with "/" as rootUri.
 		defaultUri = defaultDb.getDirectory().toURI().toString();
