@@ -54,7 +54,7 @@ public class GcBasicPackingTest extends GcTestCase {
 		assertEquals(4, stats.numberOfLooseObjects);
 		assertEquals(0, stats.numberOfPackedObjects);
 		configureGc(gc, aggressive);
-		gc.gc();
+		gc.gc().get();
 		stats = gc.getStatistics();
 		assertEquals(4, stats.numberOfLooseObjects);
 		assertEquals(0, stats.numberOfPackedObjects);
@@ -72,7 +72,7 @@ public class GcBasicPackingTest extends GcTestCase {
 		assertEquals(8, stats.numberOfLooseObjects);
 		assertEquals(0, stats.numberOfPackedObjects);
 		configureGc(gc, aggressive);
-		gc.gc();
+		gc.gc().get();
 		stats = gc.getStatistics();
 		assertEquals(0, stats.numberOfLooseObjects);
 		assertEquals(8, stats.numberOfPackedObjects);
@@ -93,7 +93,7 @@ public class GcBasicPackingTest extends GcTestCase {
 		assertEquals(8, stats.numberOfLooseObjects);
 		assertEquals(0, stats.numberOfPackedObjects);
 		configureGc(gc, aggressive);
-		gc.gc();
+		gc.gc().get();
 		stats = gc.getStatistics();
 		assertEquals(0, stats.numberOfLooseObjects);
 		assertEquals(8, stats.numberOfPackedObjects);
@@ -112,7 +112,7 @@ public class GcBasicPackingTest extends GcTestCase {
 		assertEquals(4, stats.numberOfLooseObjects);
 		assertEquals(0, stats.numberOfPackedObjects);
 		configureGc(gc, aggressive);
-		gc.gc();
+		gc.gc().get();
 		stats = gc.getStatistics();
 		assertEquals(0, stats.numberOfLooseObjects);
 		assertEquals(4, stats.numberOfPackedObjects);
@@ -120,7 +120,7 @@ public class GcBasicPackingTest extends GcTestCase {
 		assertEquals(1, stats.numberOfBitmaps);
 
 		// Do the gc again and check that it hasn't changed anything
-		gc.gc();
+		gc.gc().get();
 		stats = gc.getStatistics();
 		assertEquals(0, stats.numberOfLooseObjects);
 		assertEquals(4, stats.numberOfPackedObjects);
@@ -140,7 +140,7 @@ public class GcBasicPackingTest extends GcTestCase {
 		assertEquals(8, stats.numberOfLooseObjects);
 		assertEquals(0, stats.numberOfPackedObjects);
 		configureGc(gc, aggressive);
-		gc.gc();
+		gc.gc().get();
 		stats = gc.getStatistics();
 		assertEquals(0, stats.numberOfLooseObjects);
 		assertEquals(8, stats.numberOfPackedObjects);
@@ -168,7 +168,7 @@ public class GcBasicPackingTest extends GcTestCase {
 		gc.setExpireAgeMillis(0);
 		fsTick();
 		configureGc(gc, aggressive);
-		gc.gc();
+		gc.gc().get();
 		stats = gc.getStatistics();
 		assertEquals(0, stats.numberOfLooseObjects);
 
@@ -187,7 +187,7 @@ public class GcBasicPackingTest extends GcTestCase {
 		bb2.commit().message("M").add("M", "M").create();
 
 		gc.setExpireAgeMillis(0);
-		gc.gc();
+		gc.gc().get();
 		stats = gc.getStatistics();
 		assertEquals(0, stats.numberOfLooseObjects);
 		assertEquals(4, stats.numberOfPackedObjects);
@@ -207,7 +207,7 @@ public class GcBasicPackingTest extends GcTestCase {
 		// The old packfile is too young to be deleted. We should end up with
 		// two pack files
 		gc.setExpire(new Date(oldPackfile.lastModified() - 1));
-		gc.gc();
+		gc.gc().get();
 		stats = gc.getStatistics();
 		assertEquals(0, stats.numberOfLooseObjects);
 		// if objects exist in multiple packFiles then they are counted multiple
@@ -218,7 +218,7 @@ public class GcBasicPackingTest extends GcTestCase {
 		// repack again but now without a grace period for loose objects. Since
 		// we don't have loose objects anymore this shouldn't change anything
 		gc.setExpireAgeMillis(0);
-		gc.gc();
+		gc.gc().get();
 		stats = gc.getStatistics();
 		assertEquals(0, stats.numberOfLooseObjects);
 		// if objects exist in multiple packFiles then they are counted multiple
@@ -233,7 +233,7 @@ public class GcBasicPackingTest extends GcTestCase {
 		// we want to keep newly-loosened objects though
 		gc.setExpireAgeMillis(-1);
 
-		gc.gc();
+		gc.gc().get();
 		stats = gc.getStatistics();
 		assertEquals(1, stats.numberOfLooseObjects);
 		// if objects exist in multiple packFiles then they are counted multiple
@@ -252,7 +252,7 @@ public class GcBasicPackingTest extends GcTestCase {
 		bb2.commit().message("M").add("M", "M").create();
 
 		gc.setExpireAgeMillis(0);
-		gc.gc();
+		gc.gc().get();
 		stats = gc.getStatistics();
 
 		fsTick();
@@ -273,7 +273,7 @@ public class GcBasicPackingTest extends GcTestCase {
 		//And we don't want to keep packs full of dead objects
 		gc.setPackExpireAgeMillis(0);
 
-		gc.gc();
+		gc.gc().get();
 		stats = gc.getStatistics();
 		assertEquals(0, stats.numberOfLooseObjects);
 		assertEquals(6, stats.numberOfPackedObjects);
@@ -284,7 +284,7 @@ public class GcBasicPackingTest extends GcTestCase {
 	public void testPreserveAndPruneOldPacks() throws Exception {
 		testPreserveOldPacks();
 		configureGc(gc, false).setPrunePreserved(true);
-		gc.gc();
+		gc.gc().get();
 
 		assertFalse(repo.getObjectDatabase().getPreservedDirectory().exists());
 	}
@@ -295,7 +295,7 @@ public class GcBasicPackingTest extends GcTestCase {
 
 		// pack loose object into packfile
 		gc.setExpireAgeMillis(0);
-		gc.gc();
+		gc.gc().get();
 		PackFile oldPackfile = tr.getRepository().getObjectDatabase().getPacks()
 				.iterator().next().getPackFile();
 		assertTrue(oldPackfile.exists());
@@ -308,7 +308,7 @@ public class GcBasicPackingTest extends GcTestCase {
 		// preserved directory
 		gc.setPackExpireAgeMillis(0);
 		configureGc(gc, false).setPreserveOldPacks(true);
-		gc.gc();
+		gc.gc().get();
 
 		File preservedPackFile = oldPackfile.createPreservedForDirectory(
 				repo.getObjectDatabase().getPreservedDirectory());
@@ -330,7 +330,7 @@ public class GcBasicPackingTest extends GcTestCase {
 		configureGc(gc, false);
 		gc.setExpireAgeMillis(0);
 		gc.setPackExpireAgeMillis(0);
-		gc.gc();
+		gc.gc().get();
 		stats = gc.getStatistics();
 		assertEquals(0, stats.numberOfLooseObjects);
 		assertEquals(4, stats.numberOfPackedObjects);
@@ -347,7 +347,7 @@ public class GcBasicPackingTest extends GcTestCase {
 
 		// Repack with only orphaned commit, so packfile will be pruned
 		configureGc(gc, false).setPreserveOldPacks(true);
-		gc.gc();
+		gc.gc().get();
 		stats = gc.getStatistics();
 		assertEquals(0, stats.numberOfLooseObjects);
 		assertEquals(0, stats.numberOfPackedObjects);
