@@ -34,6 +34,7 @@ import static org.junit.Assert.assertTrue;
 import static org.junit.Assert.fail;
 
 import java.io.File;
+import java.io.FileNotFoundException;
 import java.io.IOException;
 import java.nio.file.Files;
 import java.text.MessageFormat;
@@ -50,9 +51,11 @@ import java.util.function.Consumer;
 import org.eclipse.jgit.api.MergeCommand.FastForwardMode;
 import org.eclipse.jgit.errors.ConfigInvalidException;
 import org.eclipse.jgit.internal.JGitText;
+import org.eclipse.jgit.junit.JGitTestUtil;
 import org.eclipse.jgit.junit.MockSystemReader;
 import org.eclipse.jgit.merge.MergeConfig;
 import org.eclipse.jgit.storage.file.FileBasedConfig;
+import org.eclipse.jgit.storage.file.FileRepositoryBuilder;
 import org.eclipse.jgit.transport.RefSpec;
 import org.eclipse.jgit.util.FS;
 import org.eclipse.jgit.util.SystemReader;
@@ -1174,7 +1177,7 @@ public class ConfigTest {
 		assertEquals(exp, c.getLong("s", null, "a", 0L));
 	}
 
-	private static Config parse(String content)
+	static Config parse(String content)
 			throws ConfigInvalidException {
 		return parse(content, null);
 	}
@@ -1463,6 +1466,143 @@ public class ConfigTest {
 		assertEquals("tr   ue", parseEscapedValue("tr \\\r\n  ue"));
 	}
 
+	@Test
+	public void testCommitTemplateEmptyConfig()
+			throws ConfigInvalidException, IOException {
+		// no values defined nowhere
+		Config config = new Config(null);
+		assertNull(config.get(CommitConfig.KEY).getCommitTemplatePath());
+		assertNull(config.get(CommitConfig.KEY)
+				.getCommitTemplateContent(null));
+	}
+
+	@Test
+	public void testCommitTemplateConfig()
+			throws ConfigInvalidException, IOException {
+
+		File workTree = tmp.newFolder("dummy-worktree");
+		File tempFile = tmp.newFile("testCommitTemplate-");
+		Repository repo = FileRepositoryBuilder
+				.create(new File(workTree, ".git"));
+		repo.create();
+		String templateContent = "content of the template";
+		JGitTestUtil.write(tempFile, templateContent);
+		String expectedTemplatePath = tempFile.getPath();
+
+		Config config = parse(
+				"[commit]\n\ttemplate = "
+						+ Config.escapeValue(expectedTemplatePath) + "\n");
+
+		String templatePath = config.get(CommitConfig.KEY)
+				.getCommitTemplatePath();
+		String commitEncoding = config.get(CommitConfig.KEY)
+				.getCommitEncoding();
+		assertEquals(expectedTemplatePath, templatePath);
+		assertEquals(templateContent,
+				config.get(CommitConfig.KEY).getCommitTemplateContent(repo));
+		assertNull("no commitEncoding has been set so it must be null",
+				commitEncoding);
+	}
+
+	@Test
+	public void testCommitTemplateConfigRelativePath()
+			throws ConfigInvalidException, IOException {
+
+		File workTree = tmp.newFolder("dummy-worktree");
+		File tempFile = tmp.newFile("testCommitTemplate-");
+		String templateContent = "content of the template";
+		JGitTestUtil.write(tempFile, templateContent);
+		String expectedTemplatePath = "../" + tempFile.getName();
+
+		Config config = parse(
+				"[commit]\n\ttemplate = " + expectedTemplatePath + "\n");
+
+		String templatePath = config.get(CommitConfig.KEY)
+				.getCommitTemplatePath();
+		String commitEncoding = config.get(CommitConfig.KEY)
+				.getCommitEncoding();
+		assertEquals(expectedTemplatePath, templatePath);
+		assertEquals(templateContent, config.get(CommitConfig.KEY)
+				.getCommitTemplateContent(
+						new RepositoryBuilder().setWorkTree(workTree).build()));
+		assertNull("no commitEncoding has been set so it must be null",
+				commitEncoding);
+	}
+
+	@Test
+	public void testCommitTemplateEncoding()
+			throws ConfigInvalidException, IOException {
+		Config config = new Config(null);
+		File workTree = tmp.newFolder("dummy-worktree");
+		Repository repo = FileRepositoryBuilder
+				.create(new File(workTree, ".git"));
+		repo.create();
+		File tempFile = tmp.newFile("testCommitTemplate-");
+		String templateContent = "content of the template";
+		JGitTestUtil.write(tempFile, templateContent);
+		String expectedTemplatePath = tempFile.getPath();
+		config = parse("[i18n]\n\tcommitEncoding = utf-8\n"
+				+ "[commit]\n\ttemplate = "
+				+ Config.escapeValue(expectedTemplatePath) + "\n");
+		assertEquals(templateContent,
+				config.get(CommitConfig.KEY).getCommitTemplateContent(repo));
+		String commitEncoding = config.get(CommitConfig.KEY)
+				.getCommitEncoding();
+		assertEquals("commitEncoding has been set to utf-8 it must be utf-8",
+				"utf-8", commitEncoding);
+	}
+
+	@Test(expected = ConfigInvalidException.class)
+	public void testCommitTemplateWithInvalidEncoding()
+			throws ConfigInvalidException, IOException {
+		Config config = new Config(null);
+		File workTree = tmp.newFolder("dummy-worktree");
+		File tempFile = tmp.newFile("testCommitTemplate-");
+		Repository repo = FileRepositoryBuilder
+				.create(new File(workTree, ".git"));
+		repo.create();
+		String templateContent = "content of the template";
+		JGitTestUtil.write(tempFile, templateContent);
+		config = parse("[i18n]\n\tcommitEncoding = invalidEcoding\n"
+				+ "[commit]\n\ttemplate = "
+				+ Config.escapeValue(tempFile.getPath()) + "\n");
+		config.get(CommitConfig.KEY).getCommitTemplateContent(repo);
+	}
+
+	@Test(expected = FileNotFoundException.class)
+	public void testCommitTemplateWithInvalidPath()
+			throws ConfigInvalidException, IOException {
+		Config config = new Config(null);
+		File workTree = tmp.newFolder("dummy-worktree");
+		File tempFile = tmp.newFile("testCommitTemplate-");
+		Repository repo = FileRepositoryBuilder
+				.create(new File(workTree, ".git"));
+		repo.create();
+		String templateContent = "content of the template";
+		JGitTestUtil.write(tempFile, templateContent);
+		// commit message encoding
+		String expectedTemplatePath = "~/nonExistingTemplate";
+		config = parse("[commit]\n\ttemplate = " + expectedTemplatePath + "\n");
+		String templatePath = config.get(CommitConfig.KEY)
+				.getCommitTemplatePath();
+		assertEquals(expectedTemplatePath, templatePath);
+		config.get(CommitConfig.KEY).getCommitTemplateContent(repo);
+	}
+
+	@Test
+	public void testCoreCommitGraphConfig() {
+		Config config = new Config();
+		assertFalse(config.get(CoreConfig.KEY).enableCommitGraph());
+
+		config.setBoolean(ConfigConstants.CONFIG_CORE_SECTION, null,
+				ConfigConstants.CONFIG_COMMIT_GRAPH, true);
+		assertTrue(config.get(CoreConfig.KEY).enableCommitGraph());
+
+		config.setBoolean(ConfigConstants.CONFIG_CORE_SECTION, null,
+				ConfigConstants.CONFIG_COMMIT_GRAPH, false);
+		assertFalse(config.get(CoreConfig.KEY).enableCommitGraph());
+	}
+
 	private static void assertValueRoundTrip(String value)
 			throws ConfigInvalidException {
 		assertValueRoundTrip(value, value);
