@@ -2,7 +2,7 @@
  * Copyright (C) 2007, Dave Watson <dwatson@mimvista.com>
  * Copyright (C) 2008-2011, Shawn O. Pearce <spearce@spearce.org>
  * Copyright (C) 2008-2011, Robin Rosenberg <robin.rosenberg@dewire.com>
- * Copyright (C) 2010, 2020 Christian Halstrick <christian.halstrick@sap.com> and others
+ * Copyright (C) 2010, 2022 Christian Halstrick <christian.halstrick@sap.com> and others
  *
  * This program and the accompanying materials are made available under the
  * terms of the Eclipse Distribution License v. 1.0 which is available at
@@ -242,6 +242,7 @@ public class DirCacheCheckoutTest extends RepositoryTestCase {
 		ListenerHandle handle = null;
 		try (Git git = new Git(db);
 				TestRepository<Repository> db_t = new TestRepository<>(db)) {
+			db.incrementOpen();
 			handle = db.getListenerList()
 					.addWorkingTreeModifiedListener(recorder);
 			BranchBuilder master = db_t.branch("master");
@@ -261,6 +262,7 @@ public class DirCacheCheckoutTest extends RepositoryTestCase {
 			String attributes) throws Exception {
 		try (Git git = new Git(db);
 				TestRepository<Repository> db_t = new TestRepository<>(db)) {
+			db.incrementOpen();
 			BranchBuilder master = db_t.branch("master");
 			master.commit().add("f", inIndex).message("m0").create();
 			if (!StringUtils.isEmptyOrNull(attributes)) {
@@ -313,8 +315,9 @@ public class DirCacheCheckoutTest extends RepositoryTestCase {
 
 	@Test
 	public void testCheckoutWithLFAuto() throws Exception {
-		checkoutLineEndings("first line\nsecond line\n",
-				"first line\nsecond line\n", "f text=auto");
+		String expected = String.format("first line%nsecond line%n");
+		checkoutLineEndings("first line\nsecond line\n", expected,
+				"f text=auto");
 	}
 
 	@Test
@@ -325,9 +328,9 @@ public class DirCacheCheckoutTest extends RepositoryTestCase {
 
 	@Test
 	public void testCheckoutWithLFAutoEolNative() throws Exception {
+		String expected = String.format("first line%nsecond line%n");
 		checkoutLineEndings(
-				"first line\nsecond line\n", "first line\nsecond line\n"
-						.replaceAll("\n", System.lineSeparator()),
+				"first line\nsecond line\n", expected,
 				"f text=auto eol=native");
 	}
 
@@ -337,6 +340,34 @@ public class DirCacheCheckoutTest extends RepositoryTestCase {
 				"first line\r\nsecond line\r\n", "f text=auto eol=crlf");
 	}
 
+	@Test
+	public void testCheckoutMixedAutoEolCrLf() throws Exception {
+		checkoutLineEndings("first line\nsecond line\r\n",
+				"first line\nsecond line\r\n", "f text=auto eol=crlf");
+	}
+
+	@Test
+	public void testCheckoutMixedAutoEolLf() throws Exception {
+		checkoutLineEndings("first line\nsecond line\r\n",
+				"first line\nsecond line\r\n", "f text=auto eol=lf");
+	}
+
+	@Test
+	public void testCheckoutMixedTextCrLf() throws Exception {
+		// Huh? Is this a bug in git? Both git 2.18.0 and git 2.33.0 do
+		// write the file with CRLF (and consequently report the file as
+		// modified in "git status" after check-out), however the CRLF in the
+		// repository is _not_ replaced by LF with eol=lf (see test below).
+		checkoutLineEndings("first line\nsecond line\r\n",
+				"first line\r\nsecond line\r\n", "f text eol=crlf");
+	}
+
+	@Test
+	public void testCheckoutMixedTextLf() throws Exception {
+		checkoutLineEndings("first line\nsecond line\r\nfoo",
+				"first line\nsecond line\r\nfoo", "f text eol=lf");
+	}
+
 	private DirCacheCheckout resetHard(RevCommit commit)
 			throws NoWorkTreeException,
 			CorruptObjectException, IOException {
@@ -2036,6 +2067,7 @@ public class DirCacheCheckoutTest extends RepositoryTestCase {
 	public void testCheckoutWithEmptyIndexDoesntOverwrite() throws Exception {
 		try (Git git = new Git(db);
 				TestRepository<Repository> db_t = new TestRepository<>(db)) {
+			db.incrementOpen();
 			// prepare the commits
 			BranchBuilder master = db_t.branch("master");
 			RevCommit mergeCommit = master.commit()
