@@ -1,55 +1,22 @@
 /*
- * Copyright (C) 2009, Google Inc.
- * and other copyright owners as documented in the project's IP log.
+ * Copyright (C) 2009, Google Inc. and others
  *
- * This program and the accompanying materials are made available
- * under the terms of the Eclipse Distribution License v1.0 which
- * accompanies this distribution, is reproduced below, and is
- * available at http://www.eclipse.org/org/documents/edl-v10.php
+ * This program and the accompanying materials are made available under the
+ * terms of the Eclipse Distribution License v. 1.0 which is available at
+ * https://www.eclipse.org/org/documents/edl-v10.php.
  *
- * All rights reserved.
- *
- * Redistribution and use in source and binary forms, with or
- * without modification, are permitted provided that the following
- * conditions are met:
- *
- * - Redistributions of source code must retain the above copyright
- *   notice, this list of conditions and the following disclaimer.
- *
- * - Redistributions in binary form must reproduce the above
- *   copyright notice, this list of conditions and the following
- *   disclaimer in the documentation and/or other materials provided
- *   with the distribution.
- *
- * - Neither the name of the Eclipse Foundation, Inc. nor the
- *   names of its contributors may be used to endorse or promote
- *   products derived from this software without specific prior
- *   written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
- * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
- * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
- * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
- * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
- * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
- * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
- * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
- * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ * SPDX-License-Identifier: BSD-3-Clause
  */
 
 package org.eclipse.jgit.lib;
 
 import static org.hamcrest.CoreMatchers.hasItem;
+import static org.hamcrest.MatcherAssert.assertThat;
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertFalse;
 import static org.junit.Assert.assertNotNull;
 import static org.junit.Assert.assertNotSame;
 import static org.junit.Assert.assertSame;
-import static org.junit.Assert.assertThat;
 import static org.junit.Assert.assertTrue;
 import static org.junit.Assert.fail;
 
@@ -94,24 +61,25 @@ public class RepositoryCacheTest extends RepositoryTestCase {
 
 	@Test
 	public void testFileKeyOpenExisting() throws IOException {
-		Repository r;
+		try (Repository r = new FileKey(db.getDirectory(), db.getFS())
+				.open(true)) {
+			assertNotNull(r);
+			assertEqualsFile(db.getDirectory(), r.getDirectory());
+		}
 
-		r = new FileKey(db.getDirectory(), db.getFS()).open(true);
-		assertNotNull(r);
-		assertEqualsFile(db.getDirectory(), r.getDirectory());
-		r.close();
-
-		r = new FileKey(db.getDirectory(), db.getFS()).open(false);
-		assertNotNull(r);
-		assertEqualsFile(db.getDirectory(), r.getDirectory());
-		r.close();
+		try (Repository r = new FileKey(db.getDirectory(), db.getFS())
+				.open(false)) {
+			assertNotNull(r);
+			assertEqualsFile(db.getDirectory(), r.getDirectory());
+		}
 	}
 
 	@Test
 	public void testFileKeyOpenNew() throws IOException {
-		final Repository n = createRepository(true, false);
-		final File gitdir = n.getDirectory();
-		n.close();
+		File gitdir;
+		try (Repository n = createRepository(true)) {
+			gitdir = n.getDirectory();
+		}
 		recursiveDelete(gitdir);
 		assertFalse(gitdir.exists());
 
@@ -143,6 +111,7 @@ public class RepositoryCacheTest extends RepositoryTestCase {
 	@Test
 	public void testCacheOpen() throws Exception {
 		final FileKey loc = FileKey.exact(db.getDirectory(), db.getFS());
+		@SuppressWarnings("resource") // We are testing the close() method
 		final Repository d2 = RepositoryCache.open(loc);
 		assertNotSame(db, d2);
 		assertSame(d2, RepositoryCache.open(FileKey.exact(loc.getFile(), db.getFS())));
@@ -176,6 +145,7 @@ public class RepositoryCacheTest extends RepositoryTestCase {
 	@Test
 	public void testRepositoryUsageCount() throws Exception {
 		FileKey loc = FileKey.exact(db.getDirectory(), db.getFS());
+		@SuppressWarnings("resource") // We are testing the close() method
 		Repository d2 = RepositoryCache.open(loc);
 		assertEquals(1, d2.useCnt.get());
 		RepositoryCache.open(FileKey.exact(loc.getFile(), db.getFS()));
@@ -189,6 +159,7 @@ public class RepositoryCacheTest extends RepositoryTestCase {
 	@Test
 	public void testRepositoryUsageCountWithRegisteredRepository()
 			throws IOException {
+		@SuppressWarnings({"resource", "deprecation"}) // We are testing the close() method
 		Repository repo = createRepository(false, false);
 		assertEquals(1, repo.useCnt.get());
 		RepositoryCache.register(repo);
@@ -200,6 +171,7 @@ public class RepositoryCacheTest extends RepositoryTestCase {
 	@Test
 	public void testRepositoryNotUnregisteringWhenClosing() throws Exception {
 		FileKey loc = FileKey.exact(db.getDirectory(), db.getFS());
+		@SuppressWarnings("resource") // We are testing the close() method
 		Repository d2 = RepositoryCache.open(loc);
 		assertEquals(1, d2.useCnt.get());
 		assertThat(RepositoryCache.getRegisteredKeys(),
@@ -214,6 +186,7 @@ public class RepositoryCacheTest extends RepositoryTestCase {
 	@Test
 	public void testRepositoryUnregisteringWhenExpiredAndUsageCountNegative()
 			throws Exception {
+		@SuppressWarnings("resource") // We are testing the close() method
 		Repository repoA = createBareRepository();
 		RepositoryCache.register(repoA);
 
@@ -234,7 +207,9 @@ public class RepositoryCacheTest extends RepositoryTestCase {
 
 	@Test
 	public void testRepositoryUnregisteringWhenExpired() throws Exception {
+		@SuppressWarnings({"resource", "deprecation"}) // We are testing the close() method
 		Repository repoA = createRepository(true, false);
+		@SuppressWarnings({"resource", "deprecation"}) // We are testing the close() method
 		Repository repoB = createRepository(true, false);
 		Repository repoC = createBareRepository();
 		RepositoryCache.register(repoA);
@@ -268,6 +243,7 @@ public class RepositoryCacheTest extends RepositoryTestCase {
 
 	@Test
 	public void testReconfigure() throws InterruptedException, IOException {
+		@SuppressWarnings({"resource", "deprecation"}) // We are testing the close() method
 		Repository repo = createRepository(false, false);
 		RepositoryCache.register(repo);
 		assertTrue(RepositoryCache.isCached(repo));
