@@ -241,6 +241,40 @@ public class TransportTest extends SampleDataRepositoryTestCase {
 		}
 	}
 
+	@Test
+	public void testOpenPushUseBitmaps() throws Exception {
+		URIish uri = new URIish("file://" + db.getWorkTree().getAbsolutePath());
+		// default
+		try (Transport transport = Transport.open(uri)) {
+			try (PushConnection pushConnection = transport.openPush()) {
+				assertTrue(pushConnection instanceof BasePackPushConnection);
+				@SuppressWarnings("resource")
+				BasePackPushConnection basePackPushConnection = (BasePackPushConnection) pushConnection;
+				assertTrue(basePackPushConnection.isUseBitmaps());
+			}
+		}
+		// true
+		try (Transport transport = Transport.open(uri)) {
+			transport.setPushUseBitmaps(true);
+			try (PushConnection pushConnection = transport.openPush()) {
+				assertTrue(pushConnection instanceof BasePackPushConnection);
+				@SuppressWarnings("resource")
+				BasePackPushConnection basePackPushConnection = (BasePackPushConnection) pushConnection;
+				assertTrue(basePackPushConnection.isUseBitmaps());
+			}
+		}
+		// false
+		try (Transport transport = Transport.open(uri)) {
+			transport.setPushUseBitmaps(false);
+			try (PushConnection pushConnection = transport.openPush()) {
+				assertTrue(pushConnection instanceof BasePackPushConnection);
+				@SuppressWarnings("resource")
+				BasePackPushConnection basePackPushConnection = (BasePackPushConnection) pushConnection;
+				assertFalse(basePackPushConnection.isUseBitmaps());
+			}
+		}
+	}
+
 	@Test
 	public void testSpi() {
 		List<TransportProtocol> protocols = Transport.getTransportProtocols();
