@@ -1,44 +1,11 @@
 /*
- * Copyright (C) 2008, 2015 Google Inc.
- * and other copyright owners as documented in the project's IP log.
+ * Copyright (C) 2008, 2015 Google Inc. and others
  *
- * This program and the accompanying materials are made available
- * under the terms of the Eclipse Distribution License v1.0 which
- * accompanies this distribution, is reproduced below, and is
- * available at http://www.eclipse.org/org/documents/edl-v10.php
+ * This program and the accompanying materials are made available under the
+ * terms of the Eclipse Distribution License v. 1.0 which is available at
+ * https://www.eclipse.org/org/documents/edl-v10.php.
  *
- * All rights reserved.
- *
- * Redistribution and use in source and binary forms, with or
- * without modification, are permitted provided that the following
- * conditions are met:
- *
- * - Redistributions of source code must retain the above copyright
- *   notice, this list of conditions and the following disclaimer.
- *
- * - Redistributions in binary form must reproduce the above
- *   copyright notice, this list of conditions and the following
- *   disclaimer in the documentation and/or other materials provided
- *   with the distribution.
- *
- * - Neither the name of the Eclipse Foundation, Inc. nor the
- *   names of its contributors may be used to endorse or promote
- *   products derived from this software without specific prior
- *   written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
- * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
- * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
- * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
- * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
- * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
- * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
- * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
- * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ * SPDX-License-Identifier: BSD-3-Clause
  */
 
 package org.eclipse.jgit.util;
@@ -338,7 +305,7 @@ public class NBTest {
 		assertOutput(b(0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff), out, 3);
 	}
 
-	private static void prepareOutput(final byte[] buf) {
+	private static void prepareOutput(byte[] buf) {
 		for (int i = 0; i < buf.length; i++)
 			buf[i] = (byte) (0x77 + i);
 	}
@@ -353,11 +320,11 @@ public class NBTest {
 			assertEquals((byte) (0x77 + i), buf[i]);
 	}
 
-	private static byte[] b(final int a, final int b) {
+	private static byte[] b(int a, int b) {
 		return new byte[] { (byte) a, (byte) b };
 	}
 
-	private static byte[] padb(final int len, final int a, final int b) {
+	private static byte[] padb(int len, int a, int b) {
 		final byte[] r = new byte[len + 2];
 		for (int i = 0; i < len; i++)
 			r[i] = (byte) 0xaf;
@@ -370,7 +337,7 @@ public class NBTest {
 		return new byte[] { (byte) a, (byte) b, (byte) c };
 	}
 
-	private static byte[] b(final int a, final int b, final int c, final int d) {
+	private static byte[] b(int a, int b, int c, int d) {
 		return new byte[] { (byte) a, (byte) b, (byte) c, (byte) d };
 	}
 
