@@ -1,49 +1,17 @@
 /*
- * Copyright (C) 2010, Chris Aniszczyk <caniszczyk@gmail.com>
- * and other copyright owners as documented in the project's IP log.
+ * Copyright (C) 2010, 2022 Chris Aniszczyk <caniszczyk@gmail.com> and others
  *
- * This program and the accompanying materials are made available
- * under the terms of the Eclipse Distribution License v1.0 which
- * accompanies this distribution, is reproduced below, and is
- * available at http://www.eclipse.org/org/documents/edl-v10.php
+ * This program and the accompanying materials are made available under the
+ * terms of the Eclipse Distribution License v. 1.0 which is available at
+ * https://www.eclipse.org/org/documents/edl-v10.php.
  *
- * All rights reserved.
- *
- * Redistribution and use in source and binary forms, with or
- * without modification, are permitted provided that the following
- * conditions are met:
- *
- * - Redistributions of source code must retain the above copyright
- *   notice, this list of conditions and the following disclaimer.
- *
- * - Redistributions in binary form must reproduce the above
- *   copyright notice, this list of conditions and the following
- *   disclaimer in the documentation and/or other materials provided
- *   with the distribution.
- *
- * - Neither the name of the Eclipse Foundation, Inc. nor the
- *   names of its contributors may be used to endorse or promote
- *   products derived from this software without specific prior
- *   written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
- * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
- * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
- * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
- * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
- * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
- * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
- * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
- * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ * SPDX-License-Identifier: BSD-3-Clause
  */
 package org.eclipse.jgit.api;
 
 import java.io.IOException;
 import java.io.OutputStream;
+import java.io.PrintStream;
 import java.net.URISyntaxException;
 import java.text.MessageFormat;
 import java.util.ArrayList;
@@ -54,7 +22,9 @@ import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 
+import org.eclipse.jgit.api.errors.DetachedHeadException;
 import org.eclipse.jgit.api.errors.GitAPIException;
+import org.eclipse.jgit.api.errors.InvalidRefNameException;
 import org.eclipse.jgit.api.errors.InvalidRemoteException;
 import org.eclipse.jgit.api.errors.JGitInternalException;
 import org.eclipse.jgit.errors.NotSupportedException;
@@ -62,11 +32,16 @@ import org.eclipse.jgit.errors.TooLargeObjectInPackException;
 import org.eclipse.jgit.errors.TooLargePackException;
 import org.eclipse.jgit.errors.TransportException;
 import org.eclipse.jgit.internal.JGitText;
+import org.eclipse.jgit.lib.BranchConfig;
+import org.eclipse.jgit.lib.Config;
+import org.eclipse.jgit.lib.ConfigConstants;
 import org.eclipse.jgit.lib.Constants;
 import org.eclipse.jgit.lib.NullProgressMonitor;
 import org.eclipse.jgit.lib.ProgressMonitor;
 import org.eclipse.jgit.lib.Ref;
 import org.eclipse.jgit.lib.Repository;
+import org.eclipse.jgit.transport.PushConfig;
+import org.eclipse.jgit.transport.PushConfig.PushDefault;
 import org.eclipse.jgit.transport.PushResult;
 import org.eclipse.jgit.transport.RefLeaseSpec;
 import org.eclipse.jgit.transport.RefSpec;
@@ -85,7 +60,7 @@ import org.eclipse.jgit.transport.Transport;
 public class PushCommand extends
 		TransportCommand<PushCommand, Iterable<PushResult>> {
 
-	private String remote = Constants.DEFAULT_REMOTE_NAME;
+	private String remote;
 
 	private final List<RefSpec> refSpecs;
 
@@ -99,11 +74,20 @@ public class PushCommand extends
 	private boolean atomic;
 	private boolean force;
 	private boolean thin = Transport.DEFAULT_PUSH_THIN;
+	private boolean useBitmaps = Transport.DEFAULT_PUSH_USE_BITMAPS;
+
+	private PrintStream hookOutRedirect;
+
+	private PrintStream hookErrRedirect;
 
 	private OutputStream out;
 
 	private List<String> pushOptions;
 
+	// Legacy behavior as default. Use setPushDefault(null) to determine the
+	// value from the git config.
+	private PushDefault pushDefault = PushDefault.CURRENT;
+
 	/**
 	 * <p>
 	 * Constructor for PushCommand.
@@ -131,19 +115,20 @@ public class PushCommand extends
 			InvalidRemoteException,
 			org.eclipse.jgit.api.errors.TransportException {
 		checkCallable();
+		setCallable(false);
 
 		ArrayList<PushResult> pushResults = new ArrayList<>(3);
 
 		try {
+			Config config = repo.getConfig();
+			remote = determineRemote(config, remote);
 			if (refSpecs.isEmpty()) {
-				RemoteConfig config = new RemoteConfig(repo.getConfig(),
+				RemoteConfig rc = new RemoteConfig(config,
 						getRemote());
-				refSpecs.addAll(config.getPushRefSpecs());
-			}
-			if (refSpecs.isEmpty()) {
-				Ref head = repo.exactRef(Constants.HEAD);
-				if (head != null && head.isSymbolic())
-					refSpecs.add(new RefSpec(head.getLeaf().getName()));
+				refSpecs.addAll(rc.getPushRefSpecs());
+				if (refSpecs.isEmpty()) {
+					determineDefaultRefSpecs(config);
+				}
 			}
 
 			if (force) {
@@ -151,15 +136,19 @@ public class PushCommand extends
 					refSpecs.set(i, refSpecs.get(i).setForceUpdate(true));
 			}
 
-			final List<Transport> transports;
-			transports = Transport.openAll(repo, remote, Transport.Operation.PUSH);
-			for (final Transport transport : transports) {
+			List<Transport> transports = Transport.openAll(repo, remote,
+					Transport.Operation.PUSH);
+			for (@SuppressWarnings("resource") // Explicitly closed in finally
+					final Transport transport : transports) {
 				transport.setPushThin(thin);
 				transport.setPushAtomic(atomic);
 				if (receivePack != null)
 					transport.setOptionReceivePack(receivePack);
 				transport.setDryRun(dryRun);
 				transport.setPushOptions(pushOptions);
+				transport.setPushUseBitmaps(useBitmaps);
+				transport.setHookOutputStream(hookOutRedirect);
+				transport.setHookErrorStream(hookErrRedirect);
 				configure(transport);
 
 				final Collection<RemoteRefUpdate> toPush = transport
@@ -184,8 +173,9 @@ public class PushCommand extends
 			}
 
 		} catch (URISyntaxException e) {
-			throw new InvalidRemoteException(MessageFormat.format(
-					JGitText.get().invalidRemote, remote));
+			throw new InvalidRemoteException(
+					MessageFormat.format(JGitText.get().invalidRemote, remote),
+					e);
 		} catch (TransportException e) {
 			throw new org.eclipse.jgit.api.errors.TransportException(
 					e.getMessage(), e);
@@ -202,6 +192,102 @@ public class PushCommand extends
 		return pushResults;
 	}
 
+	private String determineRemote(Config config, String remoteName)
+			throws IOException {
+		if (remoteName != null) {
+			return remoteName;
+		}
+		Ref head = repo.exactRef(Constants.HEAD);
+		String effectiveRemote = null;
+		BranchConfig branchCfg = null;
+		if (head != null && head.isSymbolic()) {
+			String currentBranch = head.getLeaf().getName();
+			branchCfg = new BranchConfig(config,
+					Repository.shortenRefName(currentBranch));
+			effectiveRemote = branchCfg.getPushRemote();
+		}
+		if (effectiveRemote == null) {
+			effectiveRemote = config.getString(
+					ConfigConstants.CONFIG_REMOTE_SECTION, null,
+					ConfigConstants.CONFIG_KEY_PUSH_DEFAULT);
+			if (effectiveRemote == null && branchCfg != null) {
+				effectiveRemote = branchCfg.getRemote();
+			}
+		}
+		if (effectiveRemote == null) {
+			effectiveRemote = Constants.DEFAULT_REMOTE_NAME;
+		}
+		return effectiveRemote;
+	}
+
+	private String getCurrentBranch()
+			throws IOException, DetachedHeadException {
+		Ref head = repo.exactRef(Constants.HEAD);
+		if (head != null && head.isSymbolic()) {
+			return head.getLeaf().getName();
+		}
+		throw new DetachedHeadException();
+	}
+
+	private void determineDefaultRefSpecs(Config config)
+			throws IOException, GitAPIException {
+		if (pushDefault == null) {
+			pushDefault = config.get(PushConfig::new).getPushDefault();
+		}
+		switch (pushDefault) {
+		case CURRENT:
+			refSpecs.add(new RefSpec(getCurrentBranch()));
+			break;
+		case MATCHING:
+			refSpecs.add(new RefSpec(":")); //$NON-NLS-1$
+			break;
+		case NOTHING:
+			throw new InvalidRefNameException(
+					JGitText.get().pushDefaultNothing);
+		case SIMPLE:
+		case UPSTREAM:
+			String currentBranch = getCurrentBranch();
+			BranchConfig branchCfg = new BranchConfig(config,
+					Repository.shortenRefName(currentBranch));
+			String fetchRemote = branchCfg.getRemote();
+			if (fetchRemote == null) {
+				fetchRemote = Constants.DEFAULT_REMOTE_NAME;
+			}
+			boolean isTriangular = !fetchRemote.equals(remote);
+			if (isTriangular) {
+				if (PushDefault.UPSTREAM.equals(pushDefault)) {
+					throw new InvalidRefNameException(MessageFormat.format(
+							JGitText.get().pushDefaultTriangularUpstream,
+							remote, fetchRemote));
+				}
+				// Strange, but consistent with C git: "simple" doesn't even
+				// check whether there is a configured upstream, and if so, that
+				// it is equal to the local branch name. It just becomes
+				// "current".
+				refSpecs.add(new RefSpec(currentBranch));
+			} else {
+				String trackedBranch = branchCfg.getMerge();
+				if (branchCfg.isRemoteLocal() || trackedBranch == null
+						|| !trackedBranch.startsWith(Constants.R_HEADS)) {
+					throw new InvalidRefNameException(MessageFormat.format(
+							JGitText.get().pushDefaultNoUpstream,
+							currentBranch));
+				}
+				if (PushDefault.SIMPLE.equals(pushDefault)
+						&& !trackedBranch.equals(currentBranch)) {
+					throw new InvalidRefNameException(MessageFormat.format(
+							JGitText.get().pushDefaultSimple, currentBranch,
+							trackedBranch));
+				}
+				refSpecs.add(new RefSpec(currentBranch + ':' + trackedBranch));
+			}
+			break;
+		default:
+			throw new InvalidRefNameException(MessageFormat
+					.format(JGitText.get().pushDefaultUnknown, pushDefault));
+		}
+	}
+
 	/**
 	 * The remote (uri or name) used for the push operation. If no remote is
 	 * set, the default value of <code>Constants.DEFAULT_REMOTE_NAME</code> will
@@ -227,6 +313,46 @@ public class PushCommand extends
 		return remote;
 	}
 
+	/**
+	 * Sets a {@link PrintStream} a "pre-push" hook may write its stdout to. If
+	 * not set, {@link System#out} will be used.
+	 * <p>
+	 * When pushing to several remote repositories the stream is shared for all
+	 * pushes.
+	 * </p>
+	 *
+	 * @param redirect
+	 *            {@link PrintStream} to use; if {@code null},
+	 *            {@link System#out} will be used
+	 * @return {@code this}
+	 * @since 6.4
+	 */
+	public PushCommand setHookOutputStream(PrintStream redirect) {
+		checkCallable();
+		hookOutRedirect = redirect;
+		return this;
+	}
+
+	/**
+	 * Sets a {@link PrintStream} a "pre-push" hook may write its stderr to. If
+	 * not set, {@link System#err} will be used.
+	 * <p>
+	 * When pushing to several remote repositories the stream is shared for all
+	 * pushes.
+	 * </p>
+	 *
+	 * @param redirect
+	 *            {@link PrintStream} to use; if {@code null},
+	 *            {@link System#err} will be used
+	 * @return {@code this}
+	 * @since 6.4
+	 */
+	public PushCommand setHookErrorStream(PrintStream redirect) {
+		checkCallable();
+		hookErrRedirect = redirect;
+		return this;
+	}
+
 	/**
 	 * The remote executable providing receive-pack service for pack transports.
 	 * If no receive-pack is set, the default value of
@@ -366,10 +492,38 @@ public class PushCommand extends
 		return this;
 	}
 
+	/**
+	 * Retrieves the {@link PushDefault} currently set.
+	 *
+	 * @return the {@link PushDefault}, or {@code null} if not set
+	 * @since 6.1
+	 */
+	public PushDefault getPushDefault() {
+		return pushDefault;
+	}
+
+	/**
+	 * Sets an explicit {@link PushDefault}. The default used if this is not
+	 * called is {@link PushDefault#CURRENT} for compatibility reasons with
+	 * earlier JGit versions.
+	 *
+	 * @param pushDefault
+	 *            {@link PushDefault} to set; if {@code null} the value defined
+	 *            in the git config will be used.
+	 *
+	 * @return {@code this}
+	 * @since 6.1
+	 */
+	public PushCommand setPushDefault(PushDefault pushDefault) {
+		checkCallable();
+		this.pushDefault = pushDefault;
+		return this;
+	}
+
 	/**
 	 * Push all branches under refs/heads/*.
 	 *
-	 * @return {code this}
+	 * @return {@code this}
 	 */
 	public PushCommand setPushAll() {
 		refSpecs.add(Transport.REFSPEC_PUSH_ALL);
@@ -379,7 +533,7 @@ public class PushCommand extends
 	/**
 	 * Push all tags under refs/tags/*.
 	 *
-	 * @return {code this}
+	 * @return {@code this}
 	 */
 	public PushCommand setPushTags() {
 		refSpecs.add(Transport.REFSPEC_TAGS);
@@ -470,6 +624,32 @@ public class PushCommand extends
 		return this;
 	}
 
+	/**
+	 * Whether to use bitmaps for push.
+	 *
+	 * @return true if push use bitmaps.
+	 * @since 6.4
+	 */
+	public boolean isUseBitmaps() {
+		return useBitmaps;
+	}
+
+	/**
+	 * Set whether to use bitmaps for push.
+	 *
+	 * Default setting is {@value Transport#DEFAULT_PUSH_USE_BITMAPS}
+	 *
+	 * @param useBitmaps
+	 *            false to disable use of bitmaps for push, true otherwise.
+	 * @return {@code this}
+	 * @since 6.4
+	 */
+	public PushCommand setUseBitmaps(boolean useBitmaps) {
+		checkCallable();
+		this.useBitmaps = useBitmaps;
+		return this;
+	}
+
 	/**
 	 * Whether this push should be executed atomically (all references updated,
 	 * or none)
