@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2009, Google Inc.
- * Copyright (C) 2008-2009, Johannes E. Schindelin <johannes.schindelin@gmx.de> and others
+ * Copyright (C) 2008-2021, Johannes E. Schindelin <johannes.schindelin@gmx.de> and others
  *
  * This program and the accompanying materials are made available under the
  * terms of the Eclipse Distribution License v. 1.0 which is available at
@@ -16,6 +16,8 @@ import java.io.File;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
+import java.nio.ByteBuffer;
+import java.util.concurrent.atomic.AtomicInteger;
 
 import org.eclipse.jgit.errors.BinaryBlobException;
 import org.eclipse.jgit.errors.LargeObjectException;
@@ -37,11 +39,20 @@ import org.eclipse.jgit.util.RawParseUtils;
  * they are converting from "line number" to "element index".
  */
 public class RawText extends Sequence {
+
 	/** A RawText of length 0 */
 	public static final RawText EMPTY_TEXT = new RawText(new byte[0]);
 
-	/** Number of bytes to check for heuristics in {@link #isBinary(byte[])} */
-	static final int FIRST_FEW_BYTES = 8000;
+	/**
+	 * Default and minimum for {@link #BUFFER_SIZE}.
+	 */
+	private static final int FIRST_FEW_BYTES = 8 * 1024;
+
+	/**
+	 * Number of bytes to check for heuristics in {@link #isBinary(byte[])}.
+	 */
+	private static final AtomicInteger BUFFER_SIZE = new AtomicInteger(
+			FIRST_FEW_BYTES);
 
 	/** The file content for this sequence. */
 	protected final byte[] content;
@@ -103,7 +114,6 @@ public class RawText extends Sequence {
 	}
 
 	/** @return total number of items in the sequence. */
-	/** {@inheritDoc} */
 	@Override
 	public int size() {
 		// The line map is always 2 entries larger than the number of lines in
@@ -164,6 +174,27 @@ public class RawText extends Sequence {
 		return getString(i, i + 1, true);
 	}
 
+	/**
+	 * Get the raw text for a single line.
+	 *
+	 * @param i
+	 *            index of the line to extract. Note this is 0-based, so line
+	 *            number 1 is actually index 0.
+	 * @return the text for the line, without a trailing LF, as a
+	 *         {@link ByteBuffer} that is backed by a slice of the
+	 *         {@link #getRawContent() raw content}, with the buffer's position
+	 *         on the start of the line and the limit at the end.
+	 * @since 5.12
+	 */
+	public ByteBuffer getRawString(int i) {
+		int s = getStart(i);
+		int e = getEnd(i);
+		if (e > 0 && content[e - 1] == '\n') {
+			e--;
+		}
+		return ByteBuffer.wrap(content, s, e - s);
+	}
+
 	/**
 	 * Get the text for a region of lines.
 	 *
@@ -214,15 +245,30 @@ public class RawText extends Sequence {
 	}
 
 	/**
-	 * Determine heuristically whether a byte array represents binary (as
-	 * opposed to text) content.
+	 * Obtains the buffer size to use for analyzing whether certain content is
+	 * text or binary, or what line endings are used if it's text.
 	 *
-	 * @param raw
-	 *            the raw file content.
-	 * @return true if raw is likely to be a binary file, false otherwise
+	 * @return the buffer size, by default {@link #FIRST_FEW_BYTES} bytes
+	 * @since 6.0
 	 */
-	public static boolean isBinary(byte[] raw) {
-		return isBinary(raw, raw.length);
+	public static int getBufferSize() {
+		return BUFFER_SIZE.get();
+	}
+
+	/**
+	 * Sets the buffer size to use for analyzing whether certain content is text
+	 * or binary, or what line endings are used if it's text. If the given
+	 * {@code bufferSize} is smaller than {@link #FIRST_FEW_BYTES} set the
+	 * buffer size to {@link #FIRST_FEW_BYTES}.
+	 *
+	 * @param bufferSize
+	 *            Size to set
+	 * @return the size actually set
+	 * @since 6.0
+	 */
+	public static int setBufferSize(int bufferSize) {
+		int newSize = Math.max(FIRST_FEW_BYTES, bufferSize);
+		return BUFFER_SIZE.updateAndGet(curr -> newSize);
 	}
 
 	/**
@@ -241,15 +287,28 @@ public class RawText extends Sequence {
 	 *             if input stream could not be read
 	 */
 	public static boolean isBinary(InputStream raw) throws IOException {
-		final byte[] buffer = new byte[FIRST_FEW_BYTES];
+		final byte[] buffer = new byte[getBufferSize() + 1];
 		int cnt = 0;
 		while (cnt < buffer.length) {
 			final int n = raw.read(buffer, cnt, buffer.length - cnt);
-			if (n == -1)
+			if (n == -1) {
 				break;
+			}
 			cnt += n;
 		}
-		return isBinary(buffer, cnt);
+		return isBinary(buffer, cnt, cnt < buffer.length);
+	}
+
+	/**
+	 * Determine heuristically whether a byte array represents binary (as
+	 * opposed to text) content.
+	 *
+	 * @param raw
+	 *            the raw file content.
+	 * @return true if raw is likely to be a binary file, false otherwise
+	 */
+	public static boolean isBinary(byte[] raw) {
+		return isBinary(raw, raw.length);
 	}
 
 	/**
@@ -265,16 +324,71 @@ public class RawText extends Sequence {
 	 * @return true if raw is likely to be a binary file, false otherwise
 	 */
 	public static boolean isBinary(byte[] raw, int length) {
-		// Same heuristic as C Git
-		if (length > FIRST_FEW_BYTES)
-			length = FIRST_FEW_BYTES;
-		for (int ptr = 0; ptr < length; ptr++)
-			if (raw[ptr] == '\0')
-				return true;
+		return isBinary(raw, length, false);
+	}
 
+	/**
+	 * Determine heuristically whether a byte array represents binary (as
+	 * opposed to text) content.
+	 *
+	 * @param raw
+	 *            the raw file content.
+	 * @param length
+	 *            number of bytes in {@code raw} to evaluate. This should be
+	 *            {@code raw.length} unless {@code raw} was over-allocated by
+	 *            the caller.
+	 * @param complete
+	 *            whether {@code raw} contains the whole data
+	 * @return true if raw is likely to be a binary file, false otherwise
+	 * @since 6.0
+	 */
+	public static boolean isBinary(byte[] raw, int length, boolean complete) {
+		// Similar heuristic as C Git. Differences:
+		// - limited buffer size; may be only the beginning of a large blob
+		// - no counting of printable vs. non-printable bytes < 0x20 and 0x7F
+		int maxLength = getBufferSize();
+		boolean isComplete = complete;
+		if (length > maxLength) {
+			// We restrict the length in all cases to getBufferSize() to get
+			// predictable behavior. Sometimes we load streams, and sometimes we
+			// have the full data in memory. With streams, we never look at more
+			// than the first getBufferSize() bytes. If we looked at more when
+			// we have the full data, different code paths in JGit might come to
+			// different conclusions.
+			length = maxLength;
+			isComplete = false;
+		}
+		byte last = 'x'; // Just something inconspicuous.
+		for (int ptr = 0; ptr < length; ptr++) {
+			byte curr = raw[ptr];
+			if (isBinary(curr, last)) {
+				return true;
+			}
+			last = curr;
+		}
+		if (isComplete) {
+			// Buffer contains everything...
+			return last == '\r'; // ... so this must be a lone CR
+		}
 		return false;
 	}
 
+	/**
+	 * Determines from the last two bytes read from a source if it looks like
+	 * binary content.
+	 *
+	 * @param curr
+	 *            the last byte, read after {@code prev}
+	 * @param prev
+	 *            the previous byte, read before {@code last}
+	 * @return {@code true} if either byte is NUL, or if prev is CR and curr is
+	 *         not LF, {@code false} otherwise
+	 * @since 6.0
+	 */
+	public static boolean isBinary(byte curr, byte prev) {
+		return curr == '\0' || (curr != '\n' && prev == '\r') || prev == '\0';
+	}
+
 	/**
 	 * Determine heuristically whether a byte array represents text content
 	 * using CR-LF as line separator.
@@ -307,7 +421,7 @@ public class RawText extends Sequence {
 	 * @since 5.3
 	 */
 	public static boolean isCrLfText(InputStream raw) throws IOException {
-		byte[] buffer = new byte[FIRST_FEW_BYTES];
+		byte[] buffer = new byte[getBufferSize()];
 		int cnt = 0;
 		while (cnt < buffer.length) {
 			int n = raw.read(buffer, cnt, buffer.length - cnt);
@@ -332,13 +446,44 @@ public class RawText extends Sequence {
 	 * @since 5.3
 	 */
 	public static boolean isCrLfText(byte[] raw, int length) {
+		return isCrLfText(raw, length, false);
+	}
+
+	/**
+	 * Determine heuristically whether a byte array represents text content
+	 * using CR-LF as line separator.
+	 *
+	 * @param raw
+	 *            the raw file content.
+	 * @param length
+	 *            number of bytes in {@code raw} to evaluate.
+	 * @return {@code true} if raw is likely to be CR-LF delimited text,
+	 *         {@code false} otherwise
+	 * @param complete
+	 *            whether {@code raw} contains the whole data
+	 * @since 6.0
+	 */
+	public static boolean isCrLfText(byte[] raw, int length, boolean complete) {
 		boolean has_crlf = false;
-		for (int ptr = 0; ptr < length - 1; ptr++) {
-			if (raw[ptr] == '\0') {
-				return false; // binary
-			} else if (raw[ptr] == '\r' && raw[ptr + 1] == '\n') {
+		byte last = 'x'; // Just something inconspicuous
+		for (int ptr = 0; ptr < length; ptr++) {
+			byte curr = raw[ptr];
+			if (isBinary(curr, last)) {
+				return false;
+			}
+			if (curr == '\n' && last == '\r') {
 				has_crlf = true;
 			}
+			last = curr;
+		}
+		if (last == '\r') {
+			if (complete) {
+				// Lone CR: it's binary after all.
+				return false;
+			}
+			// Tough call. If the next byte, which we don't have, would be a
+			// '\n', it'd be a CR-LF text, otherwise it'd be binary. Just decide
+			// based on what we already scanned; it wasn't binary until now.
 		}
 		return has_crlf;
 	}
@@ -387,18 +532,20 @@ public class RawText extends Sequence {
 			throw new BinaryBlobException();
 		}
 
-		if (sz <= FIRST_FEW_BYTES) {
-			byte[] data = ldr.getCachedBytes(FIRST_FEW_BYTES);
-			if (isBinary(data)) {
+		int bufferSize = getBufferSize();
+		if (sz <= bufferSize) {
+			byte[] data = ldr.getCachedBytes(bufferSize);
+			if (isBinary(data, data.length, true)) {
 				throw new BinaryBlobException();
 			}
 			return new RawText(data);
 		}
 
-		byte[] head = new byte[FIRST_FEW_BYTES];
+		byte[] head = new byte[bufferSize];
 		try (InputStream stream = ldr.openStream()) {
 			int off = 0;
 			int left = head.length;
+			byte last = 'x'; // Just something inconspicuous
 			while (left > 0) {
 				int n = stream.read(head, off, left);
 				if (n < 0) {
@@ -407,9 +554,11 @@ public class RawText extends Sequence {
 				left -= n;
 
 				while (n > 0) {
-					if (head[off] == '\0') {
+					byte curr = head[off];
+					if (isBinary(curr, last)) {
 						throw new BinaryBlobException();
 					}
+					last = curr;
 					off++;
 					n--;
 				}
