@@ -40,6 +40,7 @@ import org.eclipse.jgit.internal.storage.reftable.ReftableReader;
 import org.eclipse.jgit.internal.storage.reftable.ReftableWriter;
 import org.eclipse.jgit.lib.Config;
 import org.eclipse.jgit.util.FileUtils;
+import org.eclipse.jgit.util.SystemReader;
 
 /**
  * A mutable stack of reftables on local filesystem storage. Not thread-safe.
@@ -131,33 +132,6 @@ public class FileReftableStack implements AutoCloseable {
 		return stats;
 	}
 
-	/** Thrown if the update indices in the stack are not monotonic */
-	public static class ReftableNumbersNotIncreasingException
-			extends RuntimeException {
-		private static final long serialVersionUID = 1L;
-
-		String name;
-
-		long lastMax;
-
-		long min;
-
-		ReftableNumbersNotIncreasingException(String name, long lastMax,
-				long min) {
-			this.name = name;
-			this.lastMax = lastMax;
-			this.min = min;
-		}
-
-		@SuppressWarnings({ "nls", "boxing" })
-		@Override
-		public String toString() {
-			return String.format(
-					"ReftableNumbersNotIncreasingException %s: min %d, lastMax %d",
-					name, min, lastMax);
-		}
-	}
-
 	/**
 	 * Reloads the stack, potentially reusing opened reftableReaders.
 	 *
@@ -176,7 +150,6 @@ public class FileReftableStack implements AutoCloseable {
 		List<ReftableReader> newTables = new ArrayList<>();
 		List<StackEntry> newStack = new ArrayList<>(stack.size() + 1);
 		try {
-			ReftableReader last = null;
 			for (String name : names) {
 				StackEntry entry = new StackEntry();
 				entry.name = name;
@@ -194,15 +167,6 @@ public class FileReftableStack implements AutoCloseable {
 					newTables.add(t);
 				}
 
-				if (last != null) {
-					// TODO: move this to MergedReftable
-					if (last.maxUpdateIndex() >= t.minUpdateIndex()) {
-						throw new ReftableNumbersNotIncreasingException(name,
-								last.maxUpdateIndex(), t.minUpdateIndex());
-					}
-				}
-				last = t;
-
 				entry.reftableReader = t;
 				newStack.add(entry);
 			}
@@ -300,6 +264,7 @@ public class FileReftableStack implements AutoCloseable {
 		 * @param w
 		 *            writer to use
 		 * @throws IOException
+		 *             if an IO error occurred
 		 */
 		void call(ReftableWriter w) throws IOException;
 	}
@@ -564,11 +529,19 @@ public class FileReftableStack implements AutoCloseable {
 				return false;
 			}
 
+			reload();
 			for (File f : deleteOnSuccess) {
-				Files.delete(f.toPath());
+				try {
+					Files.delete(f.toPath());
+				} catch (IOException e) {
+					// Ignore: this can happen on Windows in case of concurrent processes.
+					// leave the garbage and continue.
+					if (!SystemReader.getInstance().isWindows()) {
+						throw e;
+					}
+				}
 			}
 
-			reload();
 			return true;
 		} finally {
 			if (tmpTable != null) {
@@ -587,6 +560,7 @@ public class FileReftableStack implements AutoCloseable {
 	 * Calculate an approximate log2.
 	 *
 	 * @param sz
+	 *            the number to compute an approximate log2 for
 	 * @return log2
 	 */
 	static int log(long sz) {
@@ -716,6 +690,7 @@ public class FileReftableStack implements AutoCloseable {
 	 * shape.
 	 *
 	 * @throws IOException
+	 *             if an IO error occurred
 	 */
 	private void autoCompact() throws IOException {
 		Optional<Segment> cand = autoCompactCandidate(tableSizes());
