@@ -1,7 +1,8 @@
 /*
  * Copyright (C) 2010, Mathias Kinzler <mathias.kinzler@sap.com>
  * Copyright (C) 2010, Chris Aniszczyk <caniszczyk@gmail.com>
- * Copyright (C) 2012, 2020, Robin Rosenberg and others
+ * Copyright (C) 2012-2013, Robin Rosenberg
+ * Copyright (C) 2018-2022, Andre Bossert <andre.bossert@siemens.com> and others
  *
  * This program and the accompanying materials are made available under the
  * terms of the Eclipse Distribution License v. 1.0 which is available at
@@ -9,6 +10,7 @@
  *
  * SPDX-License-Identifier: BSD-3-Clause
  */
+
 package org.eclipse.jgit.lib;
 
 /**
@@ -29,6 +31,49 @@ public final class ConfigConstants {
 	/** The "diff" section */
 	public static final String CONFIG_DIFF_SECTION = "diff";
 
+	/**
+	 * The "tool" key within "diff" or "merge" section
+	 *
+	 * @since 6.1
+	 */
+	public static final String CONFIG_KEY_TOOL = "tool";
+
+	/**
+	 * The "guitool" key within "diff" or "merge" section
+	 *
+	 * @since 6.1
+	 */
+	public static final String CONFIG_KEY_GUITOOL = "guitool";
+
+	/**
+	 * The "difftool" section
+	 *
+	 * @since 6.1
+	 */
+	public static final String CONFIG_DIFFTOOL_SECTION = "difftool";
+
+	/**
+	 * The "prompt" key within "difftool" or "mergetool" section
+	 *
+	 * @since 6.1
+	 */
+	public static final String CONFIG_KEY_PROMPT = "prompt";
+
+	/**
+	 * The "trustExitCode" key within "difftool" or "mergetool.&lt;name&gt;."
+	 * section
+	 *
+	 * @since 6.1
+	 */
+	public static final String CONFIG_KEY_TRUST_EXIT_CODE = "trustExitCode";
+
+	/**
+	 * The "cmd" key within "difftool.*." or "mergetool.*." section
+	 *
+	 * @since 6.1
+	 */
+	public static final String CONFIG_KEY_CMD = "cmd";
+
 	/** The "dfs" section */
 	public static final String CONFIG_DFS_SECTION = "dfs";
 
@@ -64,6 +109,7 @@ public final class ConfigConstants {
 
 	/**
 	 * The "fetch" section
+	 *
 	 * @since 3.3
 	 */
 	public static final String CONFIG_FETCH_SECTION = "fetch";
@@ -80,6 +126,34 @@ public final class ConfigConstants {
 	 */
 	public static final String CONFIG_MERGE_SECTION = "merge";
 
+	/**
+	 * The "mergetool" section
+	 *
+	 * @since 6.2
+	 */
+	public static final String CONFIG_MERGETOOL_SECTION = "mergetool";
+
+	/**
+	 * The "keepBackup" key within "mergetool" section
+	 *
+	 * @since 6.2
+	 */
+	public static final String CONFIG_KEY_KEEP_BACKUP = "keepBackup";
+
+	/**
+	 * The "keepTemporaries" key within "mergetool" section
+	 *
+	 * @since 6.2
+	 */
+	public static final String CONFIG_KEY_KEEP_TEMPORARIES = "keepTemporaries";
+
+	/**
+	 * The "writeToTemp" key within "mergetool" section
+	 *
+	 * @since 6.2
+	 */
+	public static final String CONFIG_KEY_WRITE_TO_TEMP = "writeToTemp";
+
 	/**
 	 * The "filter" section
 	 * @since 4.6
@@ -124,6 +198,13 @@ public final class ConfigConstants {
 	 */
 	public static final String CONFIG_COMMIT_SECTION = "commit";
 
+	/**
+	 * The "template" key
+	 *
+	 * @since 5.13
+	 */
+	public static final String CONFIG_KEY_COMMIT_TEMPLATE = "template";
+
 	/**
 	 * The "tag" section
 	 *
@@ -131,6 +212,13 @@ public final class ConfigConstants {
 	 */
 	public static final String CONFIG_TAG_SECTION = "tag";
 
+	/**
+	 * The "cleanup" key
+	 *
+	 * @since 6.1
+	 */
+	public static final String CONFIG_KEY_CLEANUP = "cleanup";
+
 	/**
 	 * The "gpgSign" key
 	 *
@@ -145,6 +233,13 @@ public final class ConfigConstants {
 	 */
 	public static final String CONFIG_KEY_FORCE_SIGN_ANNOTATED = "forceSignAnnotated";
 
+	/**
+	 * The "commentChar" key.
+	 *
+	 * @since 6.2
+	 */
+	public static final String CONFIG_KEY_COMMENT_CHAR = "commentChar";
+
 	/**
 	 * The "hooksPath" key.
 	 *
@@ -269,9 +364,29 @@ public final class ConfigConstants {
 	 */
 	public static final String CONFIG_KEY_PACKED_GIT_USE_STRONGREFS = "packedgitusestrongrefs";
 
+	/**
+	 * The "packedIndexGitUseStrongRefs" key
+	 * @since 6.7
+	 */
+	public static final String CONFIG_KEY_PACKED_INDEX_GIT_USE_STRONGREFS = "packedindexgitusestrongrefs";
+
 	/** The "remote" key */
 	public static final String CONFIG_KEY_REMOTE = "remote";
 
+	/**
+	 * The "pushRemote" key.
+	 *
+	 * @since 6.1
+	 */
+	public static final String CONFIG_KEY_PUSH_REMOTE = "pushRemote";
+
+	/**
+	 * The "pushDefault" key.
+	 *
+	 * @since 6.1
+	 */
+	public static final String CONFIG_KEY_PUSH_DEFAULT = "pushDefault";
+
 	/** The "merge" key */
 	public static final String CONFIG_KEY_MERGE = "merge";
 
@@ -341,6 +456,12 @@ public final class ConfigConstants {
 	/** The "indexversion" key */
 	public static final String CONFIG_KEY_INDEXVERSION = "indexversion";
 
+	/**
+	 * The "skiphash" key
+	 * @since 5.13.2
+	 */
+	public static final String CONFIG_KEY_SKIPHASH = "skiphash";
+
 	/**
 	 * The "hidedotfiles" key
 	 * @since 3.5
@@ -397,8 +518,16 @@ public final class ConfigConstants {
 	/** The "ff" key */
 	public static final String CONFIG_KEY_FF = "ff";
 
+	/**
+	 * The "conflictStyle" key.
+	 *
+	 * @since 5.12
+	 */
+	public static final String CONFIG_KEY_CONFLICTSTYLE = "conflictStyle";
+
 	/**
 	 * The "checkstat" key
+	 *
 	 * @since 3.0
 	 */
 	public static final String CONFIG_KEY_CHECKSTAT = "checkstat";
@@ -422,6 +551,13 @@ public final class ConfigConstants {
 	 */
 	public static final String CONFIG_KEY_SUPPORTSATOMICFILECREATION = "supportsatomicfilecreation";
 
+	/**
+	 * The "sha1Implementation" key in the "core" section
+	 *
+	 * @since 5.13.2
+	 */
+	public static final String SHA1_IMPLEMENTATION = "sha1implementation";
+
 	/**
 	 * The "noprefix" key in the "diff" section
 	 * @since 3.0
@@ -509,6 +645,13 @@ public final class ConfigConstants {
 	 */
 	public static final String CONFIG_SECTION_I18N = "i18n";
 
+	/**
+	 * The "commitEncoding" key
+	 *
+	 * @since 5.13
+	 */
+	public static final String CONFIG_KEY_COMMIT_ENCODING = "commitEncoding";
+
 	/**
 	 * The "logOutputEncoding" key
 	 *
@@ -591,6 +734,12 @@ public final class ConfigConstants {
 	 */
 	public static final String CONFIG_KEY_BITMAP_EXCESSIVE_BRANCH_COUNT = "bitmapexcessivebranchcount";
 
+	/**
+	 * The "pack.bitmapExcludedRefsPrefixes" key
+	 * @since 5.13.2
+	 */
+	public static final String CONFIG_KEY_BITMAP_EXCLUDED_REFS_PREFIXES = "bitmapexcludedrefsprefixes";
+
 	/**
 	 * The "pack.bitmapInactiveBranchAgeInDays" key
 	 * @since 5.8
@@ -603,6 +752,13 @@ public final class ConfigConstants {
 	 */
 	public static final String CONFIG_KEY_BITMAP_RECENT_COMMIT_COUNT = "bitmaprecentcommitspan";
 
+	/**
+	 * The "pack.writeReverseIndex" key
+	 *
+	 * @since 6.6
+	 */
+	public static final String CONFIG_KEY_WRITE_REVERSE_INDEX = "writeReverseIndex";
+
 	/**
 	 * The "pack.buildBitmaps" key
 	 * @since 5.8
@@ -687,6 +843,13 @@ public final class ConfigConstants {
 	 */
 	public static final String CONFIG_KEY_WINDOW_MEMORY = "windowmemory";
 
+	/**
+	 * the "pack.minBytesForObjSizeIndex" key
+	 *
+	 * @since 6.5
+	 */
+	public static final String CONFIG_KEY_MIN_BYTES_OBJ_SIZE_INDEX = "minBytesForObjSizeIndex";
+
 	/**
 	 * The "feature" section
 	 *
@@ -728,4 +891,88 @@ public final class ConfigConstants {
 	 * @since 5.11
 	 */
 	public static final String CONFIG_KEY_DEFAULT_BRANCH = "defaultbranch";
+
+	/**
+	 * The "pack.searchForReuseTimeout" key
+	 *
+	 * @since 5.13
+	 */
+	public static final String CONFIG_KEY_SEARCH_FOR_REUSE_TIMEOUT = "searchforreusetimeout";
+
+	/**
+	 * The "push" section.
+	 *
+	 * @since 6.1
+	 */
+	public static final String CONFIG_PUSH_SECTION = "push";
+
+	/**
+	 * The "default" key.
+	 *
+	 * @since 6.1
+	 */
+	public static final String CONFIG_KEY_DEFAULT = "default";
+
+	/**
+	 * The "abbrev" key
+	 *
+	 * @since 6.1
+	 */
+	public static final String CONFIG_KEY_ABBREV = "abbrev";
+
+	/**
+	 * The "writeCommitGraph" key
+	 *
+	 * @since 6.5
+	 */
+	public static final String CONFIG_KEY_WRITE_COMMIT_GRAPH = "writeCommitGraph";
+
+	/**
+	 * The "commitGraph" used by commit-graph feature
+	 *
+	 * @since 6.5
+	 */
+	public static final String CONFIG_COMMIT_GRAPH = "commitGraph";
+
+	/**
+	 * The "trustPackedRefsStat" key
+	 *
+	 * @since 6.1.1
+	 */
+	public static final String CONFIG_KEY_TRUST_PACKED_REFS_STAT = "trustPackedRefsStat";
+
+	/**
+	 * The "pack.preserveOldPacks" key
+	 *
+	 * @since 5.13.2
+	 */
+	public static final String CONFIG_KEY_PRESERVE_OLD_PACKS = "preserveoldpacks";
+
+	/**
+	 * The "pack.prunePreserved" key
+	 *
+	 * @since 5.13.2
+	 */
+	public static final String CONFIG_KEY_PRUNE_PRESERVED = "prunepreserved";
+
+	/**
+	 * The "commitGraph" section
+	 *
+	 * @since 6.7
+	 */
+	public static final String CONFIG_COMMIT_GRAPH_SECTION = "commitGraph";
+
+	/**
+	 * The "writeChangedPaths" key
+	 *
+	 * @since 6.7
+	 */
+	public static final String CONFIG_KEY_WRITE_CHANGED_PATHS = "writeChangedPaths";
+
+	/**
+	 * The "readChangedPaths" key
+	 *
+	 * @since 6.7
+	 */
+	public static final String CONFIG_KEY_READ_CHANGED_PATHS = "readChangedPaths";
 }
