@@ -1,45 +1,12 @@
 /*
  * Copyright (C) 2007, Robin Rosenberg <robin.rosenberg@dewire.com>
- * Copyright (C) 2006-2008, Shawn O. Pearce <spearce@spearce.org>
- * and other copyright owners as documented in the project's IP log.
+ * Copyright (C) 2006-2008, Shawn O. Pearce <spearce@spearce.org> and others
  *
- * This program and the accompanying materials are made available
- * under the terms of the Eclipse Distribution License v1.0 which
- * accompanies this distribution, is reproduced below, and is
- * available at http://www.eclipse.org/org/documents/edl-v10.php
+ * This program and the accompanying materials are made available under the
+ * terms of the Eclipse Distribution License v. 1.0 which is available at
+ * https://www.eclipse.org/org/documents/edl-v10.php.
  *
- * All rights reserved.
- *
- * Redistribution and use in source and binary forms, with or
- * without modification, are permitted provided that the following
- * conditions are met:
- *
- * - Redistributions of source code must retain the above copyright
- *   notice, this list of conditions and the following disclaimer.
- *
- * - Redistributions in binary form must reproduce the above
- *   copyright notice, this list of conditions and the following
- *   disclaimer in the documentation and/or other materials provided
- *   with the distribution.
- *
- * - Neither the name of the Eclipse Foundation, Inc. nor the
- *   names of its contributors may be used to endorse or promote
- *   products derived from this software without specific prior
- *   written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
- * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
- * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
- * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
- * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
- * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
- * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
- * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
- * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ * SPDX-License-Identifier: BSD-3-Clause
  */
 
 package org.eclipse.jgit.lib;
@@ -88,7 +55,8 @@ public abstract class FileMode {
 	public static final FileMode TREE = new FileMode(TYPE_TREE,
 			Constants.OBJ_TREE) {
 		@Override
-		public boolean equals(final int modeBits) {
+		@SuppressWarnings("NonOverridingEquals")
+		public boolean equals(int modeBits) {
 			return (modeBits & TYPE_MASK) == TYPE_TREE;
 		}
 	};
@@ -97,7 +65,8 @@ public abstract class FileMode {
 	public static final FileMode SYMLINK = new FileMode(TYPE_SYMLINK,
 			Constants.OBJ_BLOB) {
 		@Override
-		public boolean equals(final int modeBits) {
+		@SuppressWarnings("NonOverridingEquals")
+		public boolean equals(int modeBits) {
 			return (modeBits & TYPE_MASK) == TYPE_SYMLINK;
 		}
 	};
@@ -106,7 +75,8 @@ public abstract class FileMode {
 	public static final FileMode REGULAR_FILE = new FileMode(0100644,
 			Constants.OBJ_BLOB) {
 		@Override
-		public boolean equals(final int modeBits) {
+		@SuppressWarnings("NonOverridingEquals")
+		public boolean equals(int modeBits) {
 			return (modeBits & TYPE_MASK) == TYPE_FILE && (modeBits & 0111) == 0;
 		}
 	};
@@ -115,7 +85,8 @@ public abstract class FileMode {
 	public static final FileMode EXECUTABLE_FILE = new FileMode(0100755,
 			Constants.OBJ_BLOB) {
 		@Override
-		public boolean equals(final int modeBits) {
+		@SuppressWarnings("NonOverridingEquals")
+		public boolean equals(int modeBits) {
 			return (modeBits & TYPE_MASK) == TYPE_FILE && (modeBits & 0111) != 0;
 		}
 	};
@@ -124,7 +95,8 @@ public abstract class FileMode {
 	public static final FileMode GITLINK = new FileMode(TYPE_GITLINK,
 			Constants.OBJ_COMMIT) {
 		@Override
-		public boolean equals(final int modeBits) {
+		@SuppressWarnings("NonOverridingEquals")
+		public boolean equals(int modeBits) {
 			return (modeBits & TYPE_MASK) == TYPE_GITLINK;
 		}
 	};
@@ -133,7 +105,8 @@ public abstract class FileMode {
 	public static final FileMode MISSING = new FileMode(TYPE_MISSING,
 			Constants.OBJ_BAD) {
 		@Override
-		public boolean equals(final int modeBits) {
+		@SuppressWarnings("NonOverridingEquals")
+		public boolean equals(int modeBits) {
 			return modeBits == 0;
 		}
 	};
@@ -145,7 +118,7 @@ public abstract class FileMode {
 	 *            the mode bits the caller has somehow obtained.
 	 * @return the FileMode instance that represents the given bits.
 	 */
-	public static final FileMode fromBits(final int bits) {
+	public static final FileMode fromBits(int bits) {
 		switch (bits & TYPE_MASK) {
 		case TYPE_MISSING:
 			if (bits == 0)
@@ -165,7 +138,8 @@ public abstract class FileMode {
 
 		return new FileMode(bits, Constants.OBJ_BAD) {
 			@Override
-			public boolean equals(final int a) {
+			@SuppressWarnings("NonOverridingEquals")
+			public boolean equals(int a) {
 				return bits == a;
 			}
 		};
@@ -177,7 +151,7 @@ public abstract class FileMode {
 
 	private final int objectType;
 
-	private FileMode(int mode, final int expType) {
+	private FileMode(int mode, int expType) {
 		modeBits = mode;
 		objectType = expType;
 		if (mode != 0) {
@@ -206,7 +180,8 @@ public abstract class FileMode {
 	 *            a int.
 	 * @return true if the mode bits represent the same mode as this object
 	 */
-	public abstract boolean equals(final int modebits);
+	@SuppressWarnings("NonOverridingEquals")
+	public abstract boolean equals(int modebits);
 
 	/**
 	 * Copy this mode as a sequence of octal US-ASCII bytes.
@@ -222,7 +197,7 @@ public abstract class FileMode {
 	 * @throws java.io.IOException
 	 *             the stream encountered an error during the copy.
 	 */
-	public void copyTo(final OutputStream os) throws IOException {
+	public void copyTo(OutputStream os) throws IOException {
 		os.write(octalBytes);
 	}
 
