@@ -1,45 +1,12 @@
 /*
  * Copyright (C) 2008-2009, Google Inc.
- * Copyright (C) 2006-2008, Shawn O. Pearce <spearce@spearce.org>
- * and other copyright owners as documented in the project's IP log.
+ * Copyright (C) 2006-2008, Shawn O. Pearce <spearce@spearce.org> and others
  *
- * This program and the accompanying materials are made available
- * under the terms of the Eclipse Distribution License v1.0 which
- * accompanies this distribution, is reproduced below, and is
- * available at http://www.eclipse.org/org/documents/edl-v10.php
+ * This program and the accompanying materials are made available under the
+ * terms of the Eclipse Distribution License v. 1.0 which is available at
+ * https://www.eclipse.org/org/documents/edl-v10.php.
  *
- * All rights reserved.
- *
- * Redistribution and use in source and binary forms, with or
- * without modification, are permitted provided that the following
- * conditions are met:
- *
- * - Redistributions of source code must retain the above copyright
- *   notice, this list of conditions and the following disclaimer.
- *
- * - Redistributions in binary form must reproduce the above
- *   copyright notice, this list of conditions and the following
- *   disclaimer in the documentation and/or other materials provided
- *   with the distribution.
- *
- * - Neither the name of the Eclipse Foundation, Inc. nor the
- *   names of its contributors may be used to endorse or promote
- *   products derived from this software without specific prior
- *   written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
- * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
- * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
- * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
- * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
- * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
- * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
- * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
- * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ * SPDX-License-Identifier: BSD-3-Clause
  */
 
 package org.eclipse.jgit.util;
@@ -63,6 +30,8 @@ import java.util.HashMap;
 import java.util.Map;
 
 import org.eclipse.jgit.annotations.Nullable;
+import org.eclipse.jgit.diff.RawText;
+import org.eclipse.jgit.errors.BinaryBlobException;
 import org.eclipse.jgit.lib.Constants;
 import org.eclipse.jgit.lib.PersonIdent;
 
@@ -74,7 +43,9 @@ public final class RawParseUtils {
 	 * UTF-8 charset constant.
 	 *
 	 * @since 2.2
+	 * @deprecated use {@link java.nio.charset.StandardCharsets#UTF_8} instead
 	 */
+	@Deprecated
 	public static final Charset UTF8_CHARSET = UTF_8;
 
 	private static final byte[] digits10;
@@ -125,7 +96,7 @@ public final class RawParseUtils {
 	 *            the buffer to test for equality with b.
 	 * @return ptr + src.length if b[ptr..src.length] == src; else -1.
 	 */
-	public static final int match(final byte[] b, int ptr, final byte[] src) {
+	public static final int match(byte[] b, int ptr, byte[] src) {
 		if (ptr + src.length > b.length)
 			return -1;
 		for (int i = 0; i < src.length; i++, ptr++)
@@ -461,7 +432,7 @@ public final class RawParseUtils {
 	 *            position within buffer to start parsing digits at.
 	 * @return the timezone at this location, expressed in minutes.
 	 */
-	public static final int parseTimeZoneOffset(final byte[] b, int ptr) {
+	public static final int parseTimeZoneOffset(byte[] b, int ptr) {
 		return parseTimeZoneOffset(b, ptr, null);
 	}
 
@@ -500,7 +471,7 @@ public final class RawParseUtils {
 	 *            character to find.
 	 * @return new position just after chrA.
 	 */
-	public static final int next(final byte[] b, int ptr, final char chrA) {
+	public static final int next(byte[] b, int ptr, char chrA) {
 		final int sz = b.length;
 		while (ptr < sz) {
 			if (b[ptr++] == chrA)
@@ -520,7 +491,7 @@ public final class RawParseUtils {
 	 *            position within buffer to start looking for LF at.
 	 * @return new position just after the first LF found.
 	 */
-	public static final int nextLF(final byte[] b, int ptr) {
+	public static final int nextLF(byte[] b, int ptr) {
 		return next(b, ptr, '\n');
 	}
 
@@ -537,7 +508,7 @@ public final class RawParseUtils {
 	 *            character to find.
 	 * @return new position just after the first chrA or LF to be found.
 	 */
-	public static final int nextLF(final byte[] b, int ptr, final char chrA) {
+	public static final int nextLF(byte[] b, int ptr, char chrA) {
 		final int sz = b.length;
 		while (ptr < sz) {
 			final byte c = b[ptr++];
@@ -547,6 +518,62 @@ public final class RawParseUtils {
 		return ptr;
 	}
 
+	/**
+	 * Locate the end of the header.  Note that headers may be
+	 * more than one line long.
+	 * @param b
+	 *            buffer to scan.
+	 * @param ptr
+	 *            position within buffer to start looking for the end-of-header.
+	 * @return new position just after the header.  This is either
+	 * b.length, or the index of the header's terminating newline.
+	 * @since 5.1
+	 */
+	public static final int headerEnd(final byte[] b, int ptr) {
+		final int sz = b.length;
+		while (ptr < sz) {
+			final byte c = b[ptr++];
+			if (c == '\n' && (ptr == sz || b[ptr] != ' ')) {
+				return ptr - 1;
+			}
+		}
+		return ptr - 1;
+	}
+
+	/**
+	 * Find the start of the contents of a given header.
+	 *
+	 * @param b
+	 *            buffer to scan.
+	 * @param headerName
+	 *            header to search for
+	 * @param ptr
+	 *            position within buffer to start looking for header at.
+	 * @return new position at the start of the header's contents, -1 for
+	 *         not found
+	 * @since 5.1
+	 */
+	public static final int headerStart(byte[] headerName, byte[] b, int ptr) {
+		// Start by advancing to just past a LF or buffer start
+		if (ptr != 0) {
+			ptr = nextLF(b, ptr - 1);
+		}
+		while (ptr < b.length - (headerName.length + 1)) {
+			boolean found = true;
+			for (byte element : headerName) {
+				if (element != b[ptr++]) {
+					found = false;
+					break;
+				}
+			}
+			if (found && b[ptr++] == ' ') {
+				return ptr;
+			}
+			ptr = nextLF(b, ptr);
+		}
+		return -1;
+	}
+
 	/**
 	 * Locate the first position before a given character.
 	 *
@@ -558,7 +585,7 @@ public final class RawParseUtils {
 	 *            character to find.
 	 * @return new position just before chrA, -1 for not found
 	 */
-	public static final int prev(final byte[] b, int ptr, final char chrA) {
+	public static final int prev(byte[] b, int ptr, char chrA) {
 		if (ptr == b.length)
 			--ptr;
 		while (ptr >= 0) {
@@ -579,7 +606,7 @@ public final class RawParseUtils {
 	 *            position within buffer to start looking for LF at.
 	 * @return new position just before the first LF found, -1 for not found
 	 */
-	public static final int prevLF(final byte[] b, int ptr) {
+	public static final int prevLF(byte[] b, int ptr) {
 		return prev(b, ptr, '\n');
 	}
 
@@ -597,7 +624,7 @@ public final class RawParseUtils {
 	 * @return new position just before the first chrA or LF to be found, -1 for
 	 *         not found
 	 */
-	public static final int prevLF(final byte[] b, int ptr, final char chrA) {
+	public static final int prevLF(byte[] b, int ptr, char chrA) {
 		if (ptr == b.length)
 			--ptr;
 		while (ptr >= 0) {
@@ -620,10 +647,6 @@ public final class RawParseUtils {
 	 * <p>
 	 * The last element (index <code>map.size()-1</code>) always contains
 	 * <code>end</code>.
-	 * <p>
-	 * If the data contains a '\0' anywhere, the whole region is considered
-	 * binary and a LineMap corresponding to a single line is returned.
-	 * </p>
 	 *
 	 * @param buf
 	 *            buffer to scan.
@@ -632,31 +655,55 @@ public final class RawParseUtils {
 	 *            line 1.
 	 * @param end
 	 *            1 past the end of the content within <code>buf</code>.
-	 * @return a line map indexing the start position of each line.
+	 * @return a line map indicating the starting position of each line.
 	 */
-	public static final IntList lineMap(final byte[] buf, int ptr, int end) {
-		int start = ptr;
+	public static final IntList lineMap(byte[] buf, int ptr, int end) {
+		IntList map = new IntList((end - ptr) / 36);
+		map.fillTo(1, Integer.MIN_VALUE);
+		for (; ptr < end; ptr = nextLF(buf, ptr)) {
+			map.add(ptr);
+		}
+		map.add(end);
+		return map;
+	}
 
+	/**
+	 * Like {@link #lineMap(byte[], int, int)} but throw
+	 * {@link BinaryBlobException} if a NUL byte is encountered.
+	 *
+	 * @param buf
+	 *            buffer to scan.
+	 * @param ptr
+	 *            position within the buffer corresponding to the first byte of
+	 *            line 1.
+	 * @param end
+	 *            1 past the end of the content within <code>buf</code>.
+	 * @return a line map indicating the starting position of each line.
+	 * @throws BinaryBlobException
+	 *             if a NUL byte or a lone CR is found.
+	 * @since 5.0
+	 */
+	public static final IntList lineMapOrBinary(byte[] buf, int ptr, int end)
+			throws BinaryBlobException {
 		// Experimentally derived from multiple source repositories
 		// the average number of bytes/line is 36. Its a rough guess
 		// to initially size our map close to the target.
 		IntList map = new IntList((end - ptr) / 36);
 		map.add(Integer.MIN_VALUE);
-		boolean foundLF = true;
+		byte last = '\n'; // Must be \n to add the initial ptr
 		for (; ptr < end; ptr++) {
-			if (foundLF) {
+			if (last == '\n') {
 				map.add(ptr);
 			}
-
-			if (buf[ptr] == '\0') {
-				// binary data.
-				map = new IntList(3);
-				map.add(Integer.MIN_VALUE);
-				map.add(start);
-				break;
+			byte curr = buf[ptr];
+			if (RawText.isBinary(curr, last)) {
+				throw new BinaryBlobException();
 			}
-
-			foundLF = (buf[ptr] == '\n');
+			last = curr;
+		}
+		if (last == '\r') {
+			// Counts as binary
+			throw new BinaryBlobException();
 		}
 		map.add(end);
 		return map;
@@ -675,7 +722,7 @@ public final class RawParseUtils {
 	 *         character of the author's name. If no author header can be
 	 *         located -1 is returned.
 	 */
-	public static final int author(final byte[] b, int ptr) {
+	public static final int author(byte[] b, int ptr) {
 		final int sz = b.length;
 		if (ptr == 0)
 			ptr += 46; // skip the "tree ..." line.
@@ -697,7 +744,7 @@ public final class RawParseUtils {
 	 *         character of the committer's name. If no committer header can be
 	 *         located -1 is returned.
 	 */
-	public static final int committer(final byte[] b, int ptr) {
+	public static final int committer(byte[] b, int ptr) {
 		final int sz = b.length;
 		if (ptr == 0)
 			ptr += 46; // skip the "tree ..." line.
@@ -721,7 +768,7 @@ public final class RawParseUtils {
 	 *         character of the tagger's name. If no tagger header can be
 	 *         located -1 is returned.
 	 */
-	public static final int tagger(final byte[] b, int ptr) {
+	public static final int tagger(byte[] b, int ptr) {
 		final int sz = b.length;
 		if (ptr == 0)
 			ptr += 48; // skip the "object ..." line.
@@ -749,7 +796,7 @@ public final class RawParseUtils {
 	 *         character of the encoding's name. If no encoding header can be
 	 *         located -1 is returned (and UTF-8 should be assumed).
 	 */
-	public static final int encoding(final byte[] b, int ptr) {
+	public static final int encoding(byte[] b, int ptr) {
 		final int sz = b.length;
 		while (ptr < sz) {
 			if (b[ptr] == '\n')
@@ -773,7 +820,7 @@ public final class RawParseUtils {
 	 * @since 4.2
 	 */
 	@Nullable
-	public static String parseEncodingName(final byte[] b) {
+	public static String parseEncodingName(byte[] b) {
 		int enc = encoding(b, 0);
 		if (enc < 0) {
 			return null;
@@ -801,7 +848,7 @@ public final class RawParseUtils {
 	 *             if the JRE does not support the character set requested by
 	 *             the encoding header.
 	 */
-	public static Charset parseEncoding(final byte[] b) {
+	public static Charset parseEncoding(byte[] b) {
 		String enc = parseEncodingName(b);
 		if (enc == null) {
 			return UTF_8;
@@ -820,6 +867,26 @@ public final class RawParseUtils {
 		}
 	}
 
+	/**
+	 * Parse the "encoding " header into a character set reference.
+	 * <p>
+	 * If unsuccessful, return UTF-8.
+	 *
+	 * @param buffer
+	 *            buffer to scan.
+	 * @return the Java character set representation. Never null. Default to
+	 *         UTF-8.
+	 * @see #parseEncoding(byte[])
+	 * @since 6.7
+	 */
+	public static Charset guessEncoding(byte[] buffer) {
+		try {
+			return parseEncoding(buffer);
+		} catch (IllegalCharsetNameException | UnsupportedCharsetException e) {
+			return UTF_8;
+		}
+	}
+
 	/**
 	 * Parse a name string (e.g. author, committer, tagger) into a PersonIdent.
 	 * <p>
@@ -831,7 +898,7 @@ public final class RawParseUtils {
 	 * @return the parsed identity or null in case the identity could not be
 	 *         parsed.
 	 */
-	public static PersonIdent parsePersonIdent(final String in) {
+	public static PersonIdent parsePersonIdent(String in) {
 		return parsePersonIdent(Constants.encode(in), 0);
 	}
 
@@ -853,7 +920,7 @@ public final class RawParseUtils {
 	 * @return the parsed identity or null in case the identity could not be
 	 *         parsed.
 	 */
-	public static PersonIdent parsePersonIdent(final byte[] raw, final int nameB) {
+	public static PersonIdent parsePersonIdent(byte[] raw, int nameB) {
 		Charset cs;
 		try {
 			cs = parseEncoding(raw);
@@ -961,7 +1028,7 @@ public final class RawParseUtils {
 	 * @return position of the ':' which terminates the footer line key if this
 	 *         is otherwise a valid footer line key; otherwise -1.
 	 */
-	public static int endOfFooterLineKey(final byte[] raw, int ptr) {
+	public static int endOfFooterLineKey(byte[] raw, int ptr) {
 		try {
 			for (;;) {
 				final byte c = raw[ptr];
@@ -988,7 +1055,7 @@ public final class RawParseUtils {
 	 * @return a string representation of the range <code>[start,end)</code>,
 	 *         after decoding the region through the specified character set.
 	 */
-	public static String decode(final byte[] buffer) {
+	public static String decode(byte[] buffer) {
 		return decode(buffer, 0, buffer.length);
 	}
 
@@ -1026,7 +1093,7 @@ public final class RawParseUtils {
 	 * @return a string representation of the range <code>[start,end)</code>,
 	 *         after decoding the region through the specified character set.
 	 */
-	public static String decode(final Charset cs, final byte[] buffer) {
+	public static String decode(Charset cs, byte[] buffer) {
 		return decode(cs, buffer, 0, buffer.length);
 	}
 
@@ -1108,7 +1175,7 @@ public final class RawParseUtils {
 
 		// Try the default character set. A small group of people
 		// might actually use the same (or very similar) locale.
-		Charset defcs = Charset.defaultCharset();
+		Charset defcs = SystemReader.getInstance().getDefaultCharset();
 		if (!defcs.equals(cs) && !defcs.equals(UTF_8)) {
 			try {
 				return decode(b, defcs);
@@ -1143,7 +1210,7 @@ public final class RawParseUtils {
 		return r.toString();
 	}
 
-	private static String decode(final ByteBuffer b, final Charset charset)
+	private static String decode(ByteBuffer b, Charset charset)
 			throws CharacterCodingException {
 		final CharsetDecoder d = charset.newDecoder();
 		d.onMalformedInput(CodingErrorAction.REPORT);
@@ -1162,7 +1229,7 @@ public final class RawParseUtils {
 	 *            commit buffer.
 	 * @return position of the user's message buffer.
 	 */
-	public static final int commitMessage(final byte[] b, int ptr) {
+	public static final int commitMessage(byte[] b, int ptr) {
 		final int sz = b.length;
 		if (ptr == 0)
 			ptr += 46; // skip the "tree ..." line.
@@ -1186,7 +1253,7 @@ public final class RawParseUtils {
 	 *            buffer.
 	 * @return position of the user's message buffer.
 	 */
-	public static final int tagMessage(final byte[] b, int ptr) {
+	public static final int tagMessage(byte[] b, int ptr) {
 		final int sz = b.length;
 		if (ptr == 0)
 			ptr += 48; // skip the "object ..." line.
@@ -1211,7 +1278,7 @@ public final class RawParseUtils {
 	 * @return position of the LF at the end of the paragraph;
 	 *         <code>b.length</code> if no paragraph end could be located.
 	 */
-	public static final int endOfParagraph(final byte[] b, final int start) {
+	public static final int endOfParagraph(byte[] b, int start) {
 		int ptr = start;
 		final int sz = b.length;
 		while (ptr < sz && (b[ptr] != '\n' && b[ptr] != '\r'))
