@@ -10,6 +10,7 @@
 
 package org.eclipse.jgit.util.io;
 
+import java.io.BufferedOutputStream;
 import java.io.IOException;
 import java.io.OutputStream;
 
@@ -20,18 +21,17 @@ import org.eclipse.jgit.diff.RawText;
  *
  * Existing CRLF are not expanded to CRCRLF, but retained as is.
  *
- * A binary check on the first 8000 bytes is performed and in case of binary
- * files, canonicalization is turned off (for the complete file).
+ * A binary check on the first {@link RawText#getBufferSize()} bytes is
+ * performed and in case of binary files, canonicalization is turned off (for
+ * the complete file).
  */
 public class AutoCRLFOutputStream extends OutputStream {
 
-	static final int BUFFER_SIZE = 8000;
-
 	private final OutputStream out;
 
 	private int buf = -1;
 
-	private byte[] binbuf = new byte[BUFFER_SIZE];
+	private byte[] binbuf = new byte[RawText.getBufferSize()];
 
 	private byte[] onebytebuf = new byte[1];
 
@@ -59,18 +59,18 @@ public class AutoCRLFOutputStream extends OutputStream {
 	 * @since 4.3
 	 */
 	public AutoCRLFOutputStream(OutputStream out, boolean detectBinary) {
-		this.out = out;
+		// avoid to write single lines directly to FileOutputStream:
+		this.out = out instanceof BufferedOutputStream ? out
+				: new BufferedOutputStream(out);
 		this.detectBinary = detectBinary;
 	}
 
-	/** {@inheritDoc} */
 	@Override
 	public void write(int b) throws IOException {
 		onebytebuf[0] = (byte) b;
 		write(onebytebuf, 0, 1);
 	}
 
-	/** {@inheritDoc} */
 	@Override
 	public void write(byte[] b) throws IOException {
 		int overflow = buffer(b, 0, b.length);
@@ -78,7 +78,6 @@ public class AutoCRLFOutputStream extends OutputStream {
 			write(b, b.length - overflow, overflow);
 	}
 
-	/** {@inheritDoc} */
 	@Override
 	public void write(byte[] b, int startOff, int startLen)
 			throws IOException {
@@ -123,20 +122,25 @@ public class AutoCRLFOutputStream extends OutputStream {
 	}
 
 	private int buffer(byte[] b, int off, int len) throws IOException {
-		if (binbufcnt > binbuf.length)
+		if (binbufcnt > binbuf.length) {
 			return len;
+		}
 		int copy = Math.min(binbuf.length - binbufcnt, len);
 		System.arraycopy(b, off, binbuf, binbufcnt, copy);
 		binbufcnt += copy;
 		int remaining = len - copy;
-		if (remaining > 0)
-			decideMode();
+		if (remaining > 0) {
+			decideMode(false);
+		}
 		return remaining;
 	}
 
-	private void decideMode() throws IOException {
+	private void decideMode(boolean complete) throws IOException {
 		if (detectBinary) {
-			isBinary = RawText.isBinary(binbuf, binbufcnt);
+			isBinary = RawText.isBinary(binbuf, binbufcnt, complete);
+			if (!isBinary) {
+				isBinary = RawText.isCrLfText(binbuf, binbufcnt, complete);
+			}
 			detectBinary = false;
 		}
 		int cachedLen = binbufcnt;
@@ -144,16 +148,15 @@ public class AutoCRLFOutputStream extends OutputStream {
 		write(binbuf, 0, cachedLen);
 	}
 
-	/** {@inheritDoc} */
 	@Override
 	public void flush() throws IOException {
-		if (binbufcnt <= binbuf.length)
-			decideMode();
+		if (binbufcnt <= binbuf.length) {
+			decideMode(true);
+		}
 		buf = -1;
 		out.flush();
 	}
 
-	/** {@inheritDoc} */
 	@Override
 	public void close() throws IOException {
 		flush();
