@@ -22,23 +22,21 @@ import org.eclipse.jgit.diff.RawText;
  * Existing single CR are not changed to LF, but retained as is.
  * </p>
  * <p>
- * A binary check on the first 8000 bytes is performed and in case of binary
- * files, canonicalization is turned off (for the complete file). If the binary
- * check determines that the input is not binary but text with CR/LF,
- * canonicalization is also turned off.
+ * A binary check on the first {@link RawText#getBufferSize()} bytes is
+ * performed and in case of binary files, canonicalization is turned off (for
+ * the complete file). If the binary check determines that the input is not
+ * binary but text with CR/LF, canonicalization is also turned off.
  * </p>
  *
  * @since 4.3
  */
 public class AutoLFOutputStream extends OutputStream {
 
-	static final int BUFFER_SIZE = 8000;
-
 	private final OutputStream out;
 
 	private int buf = -1;
 
-	private byte[] binbuf = new byte[BUFFER_SIZE];
+	private byte[] binbuf = new byte[RawText.getBufferSize()];
 
 	private byte[] onebytebuf = new byte[1];
 
@@ -71,14 +69,12 @@ public class AutoLFOutputStream extends OutputStream {
 		this.detectBinary = detectBinary;
 	}
 
-	/** {@inheritDoc} */
 	@Override
 	public void write(int b) throws IOException {
 		onebytebuf[0] = (byte) b;
 		write(onebytebuf, 0, 1);
 	}
 
-	/** {@inheritDoc} */
 	@Override
 	public void write(byte[] b) throws IOException {
 		int overflow = buffer(b, 0, b.length);
@@ -87,7 +83,6 @@ public class AutoLFOutputStream extends OutputStream {
 		}
 	}
 
-	/** {@inheritDoc} */
 	@Override
 	public void write(byte[] b, int startOff, int startLen)
 			throws IOException {
@@ -148,16 +143,16 @@ public class AutoLFOutputStream extends OutputStream {
 		binbufcnt += copy;
 		int remaining = len - copy;
 		if (remaining > 0) {
-			decideMode();
+			decideMode(false);
 		}
 		return remaining;
 	}
 
-	private void decideMode() throws IOException {
+	private void decideMode(boolean complete) throws IOException {
 		if (detectBinary) {
-			isBinary = RawText.isBinary(binbuf, binbufcnt);
+			isBinary = RawText.isBinary(binbuf, binbufcnt, complete);
 			if (!isBinary) {
-				isBinary = RawText.isCrLfText(binbuf, binbufcnt);
+				isBinary = RawText.isCrLfText(binbuf, binbufcnt, complete);
 			}
 			detectBinary = false;
 		}
@@ -166,16 +161,14 @@ public class AutoLFOutputStream extends OutputStream {
 		write(binbuf, 0, cachedLen);
 	}
 
-	/** {@inheritDoc} */
 	@Override
 	public void flush() throws IOException {
 		if (binbufcnt <= binbuf.length) {
-			decideMode();
+			decideMode(true);
 		}
 		out.flush();
 	}
 
-	/** {@inheritDoc} */
 	@Override
 	public void close() throws IOException {
 		flush();
